#!/usr/bin/python

import logging
from socket import gaierror
import threading
from pyModbusTCP.client import ModbusClient
import sys
import os
from colr import color
import time

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from mypvdevices.ModbusDevice import ModbusDevice
from mypvdevices.DeviceDiscoverer import DeviceDiscoverer

RESOLVEHOSTERROR = 99

class ModbusTcpException(Exception):
    def __init__(self, msg, code):

        if code == None:
            msg="errorcode required"
            raise TypeError(msg)

        self.code = code
        self.message = msg
    def __str__(self):
        return repr(str(self.message) + ". Error-Code: " + str(self.code))

class ModbusTcpDevice(ModbusDevice):
    __devicetype__ = "ModbusTcpDevice"
    __mutex__ = threading.Lock()
    __modbusClient__ = None
    __host__ = None
    __modbushosterror__ = False
    __modbusconnectionerrorcounter__ = 0
    __modbusconnectioncounter__ = 0

    def __init__(self, serial):
        ModbusDevice.__init__(self, serial)
        self.recoverhost()

    def getidentifier(self):
        return self.getserial() + ", Host: " + str(self.__host__)

    def sethost(self, hostname):
        logging.info(self.getidentifier() + " Setting host to " + str(hostname))

        if hostname == None:
            msg="hostname required"
            logging.debug(self.getidentifier() + ": " + str(msg))
            raise TypeError(msg)

        if not isinstance(hostname, str):
            msg="hostname hast to be a string"
            logging.debug(self.getidentifier() + ": " + str(msg))
            raise TypeError(msg)

        self.__host__ = hostname

        if self.__modbusClient__ == None:
            try:
                self.__modbusClient__ = ModbusClient(host=self.__host__, port=502, timeout=5, auto_open=True, auto_close=True)
            except ValueError:
                print("Error with host or port params")
        else:
            self.__modbusClient__.host(self.__host__)

    def __readregisters__(self, startregisteraddress, registerstoread):
        with self.__mutex__:
            time.sleep(0.5)
            if self.__modbusClient__ != None:
                logging.debug(self.getidentifier() + ": Reading " + str(registerstoread) +" registers starting with register "+ str(startregisteraddress))
                registers = dict()
                try:
                    self.__modbusconnectioncounter__ += 1
                    modbus_response = self.__modbusClient__.read_holding_registers(startregisteraddress, registerstoread)
                    self.__modbushosterror__ = False
                    if modbus_response == None:
                        errorcode = self.__modbusClient__.last_error()

                        if errorcode == 2:
                            raise ModbusTcpException("Host not reachable " + str(self.__modbusClient__.host()), errorcode)
                        elif errorcode == 4:
                            raise ModbusTcpException("Invalid Registers " + str(startregisteraddress) + " " + str(registerstoread), errorcode)
                        else:
                            raise ModbusTcpException("Unknown Error. Error Code " + str(errorcode), errorcode)

                    logging.debug(self.getidentifier() + ": Data: " + str(modbus_response))
                    for i in range(len(modbus_response)):
                        registers[startregisteraddress + i] = modbus_response[i]
                except gaierror as e:
                    logging.warning(self.getidentifier() + " Cannot resolve host: " + str(self.__modbusClient__.host()))
                    self.__modbushosterror__ = True
                    self.__increaseModbusErrorCounter__()
                    raise ModbusTcpException("Cannot resolve host. " + str(self.__modbusClient__.host()), RESOLVEHOSTERROR)
                
                except Exception as e:
                    logging.warning(self.getidentifier() + ": Host: " + str(self.__modbusClient__.host()) + " Modbus read error: " + str(e))
                    self.__increaseModbusErrorCounter__()
                    raise e

                return registers

            else:
                logging.error(self.getidentifier() + ": no Modbus-Client ")
                return None

    def __readregister__(self, registeraddress):
        with self.__mutex__:
            time.sleep(0.5)
            if self.__modbusClient__ != None:
                logging.debug(self.getidentifier() + ": Reading register " + str(registeraddress))
                try:
                    self.__modbusconnectioncounter__ += 1
                    modbus_response = self.__modbusClient__.read_holding_registers(registeraddress, 1)
                    self.__modbushosterror__ = False
                    if modbus_response == None:
                        errorcode = self.__modbusClient__.last_error()

                        if errorcode == 2:
                            raise ModbusTcpException("Host not reachable " + str(self.__modbusClient__.host()), errorcode)
                        elif errorcode == 4:
                            raise ModbusTcpException("Invalid Register " + str(registeraddress), errorcode)
                        else:
                            raise ModbusTcpException("Unknown Error. Error Code " + str(errorcode), errorcode)

                    logging.debug(self.getidentifier() + ": Data: " + str(modbus_response))
                    register = modbus_response[0]
                    return register

                except gaierror as e:
                    logging.warning(self.getidentifier() + " Cannot resolve host: " + str(self.__modbusClient__.host()))
                    self.__modbushosterror__ = True
                    self.__increaseModbusErrorCounter__()
                    raise ModbusTcpException("Cannot resolve host. " + str(self.__modbusClient__.host()), RESOLVEHOSTERROR)
                
                except Exception as e:
                    logging.warning(self.getidentifier() + ". Modbus error: " + str(e))
                    self.__increaseModbusErrorCounter__()
                    raise e
            else:
                logging.error(self.getidentifier() + ": no Modbus-Client ")
                raise Exception("Register " + str(registeraddress) + " cannot be read. No modbus-client.")

    def __writeregister__(self, registerId, value):
        with self.__mutex__:
            time.sleep(0.5)
            if self.__modbusClient__ != None:
                try:
                    self.__modbusconnectioncounter__ += 1
                    result = self.__modbusClient__.write_single_register(registerId, value)
                    self.__modbushosterror__ = False
                except gaierror as e:
                    logging.warning(self.getidentifier() + " Cannot resolve host: " + str(self.__modbusClient__.host()))
                    self.__modbushosterror__ = True
                    self.__increaseModbusErrorCounter__()
                    raise ModbusTcpException("Cannot resolve host. " + str(self.__modbusClient__.host()), RESOLVEHOSTERROR)
                except Exception as e:
                    logging.warning(self.getidentifier() + " Modbus Write Error: " + str(e))
                    self.__increaseModbusErrorCounter__()
                    raise e
                if result == None:
                    logging.warning(self.getidentifier() + " Cannot write register. Register " + str(registerId))
                    self.__increaseModbusErrorCounter__()
                    raise ModbusTcpException("Cannot write register " + str(registerId), 22)    #Todo errornr

    def __supervise__(self):
        ModbusDevice.__supervise__(self)
        if self.__modbushosterror__:
            if not self.recoverhost():
                logging.warning(self.getidentifier() + " Cannot rediscover host.")
                    
    def getcommunicationerrorscounter(self):
        return self.__modbusconnectionerrorcounter__

    def getcommunicationerrorsrate(self):
        with self.__mutex__:
            errorRate = None
            try:
                errorRate = round(self.__modbusconnectionerrorcounter__ / self.__modbusconnectioncounter__, 2)
            except:
                logging.debug(self.getidentifier() + " Cannot calculate modbus tcp error rate.")
            return errorRate

    def resetCounters(self, nodeId):
        with self.__mutex__:
            try:
                self.__modbusconnectionerrorcounter__ = 0
                self.__modbusconnectioncounter__ = 0
            except Exception as e:
                logging.error(self.getidentifier() + " Error resetting modbus tcp connection counters. " + str(e))

    def __increaseModbusErrorCounter__(self):
        self.__modbusconnectionerrorcounter__ += self.__modbusconnectionerrorcounter__
        logging.debug(self.getidentifier() + " Communication error. Total errors: " + str(self.__modbusconnectionerrorcounter__))
        return self.__modbusconnectionerrorcounter__

    def discoverDevice(self):
        try:
            return DeviceDiscoverer.instance().getipforserial(self.__serial__)
        except Exception as e:
            logging.warning(self.getidentifier() + " Cannot discover device " + str(self.__serial__) + ". Error: " + str(e))
            return None

    def recoverhost(self):
        host = self.discoverDevice()
        if host == None:
            logging.warning(self.getidentifier() + " Cannot recover host.")
            return False
        else:
            self.sethost(host)
            return True

    def __getregistermapping__(self):
        datasets = {}
        datasets["power"] = self.__createDataset__(1000, "sum")
        datasets["hour"] = self.__createDataset__(1009, "avg")
        datasets["temp1"] = self.__createDataset__(1001, "avg")
        return datasets

    def __createsetup__(self):
        return {
            "device": self.__devicetype__,
            "fwversion": self.__firmwareversion__,
            "serialno": self.__serial__,
            "boost_mode": 0,
            "test": 33
            }

    def __getsettingsmap__(self):
        settings = {
            "boost_mode": {
                "register": 1005,
                "forced": True
            },
            "test": {
                "register": 1018,
                "forced": True
            }
        }
        return settings

# Entry Point     
if __name__ == "__main__":

    from DcsConnection import DcsConnection

    logging.basicConfig(format='%(asctime)s %(levelname)s[%(threadName)s:%(module)s|%(funcName)s]: %(message)s', level=logging.INFO)

    # serial = "1601242002040015"
    serial = "2001002006100016"
    serial_fake = "2001005006100000"
    correctip = "192.168.92.29"

    #device connection tests
    dcsserial = "120100200505tes1"
    cryptoKey = "41424142414241424142414241424142"
    server = "my-pv.live"


    # #Test
    # device = ModbusTcpDevice(serial)
    # register = device.readregister(1001)
    # # device.sethost("seppi")
    # device.sethost("192.168.92.254")
    # register = device.readregister(1001)

    # connection = DcsConnection(dcsserial, cryptoKey, server, 50333)
    # # device.addconnection(connection)
    # # device.setLogDataSendInterval(5)
    # device.start()
    # try:
    #     while True:
    #         print(color('[ModbusTcpDevice] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
    #         device.showinfo()
    #         device.showcommunicationerrors()
    #         device.showcommunicationerrorsrate()
    #         time.sleep(10)
    # except KeyboardInterrupt as e:
    #     print("[DCS-Connection] Stopping Test...")
    #     device.stop()



    #Constructor Tests
    try:
        device = ModbusTcpDevice("123456789")
        print(color('ERROR: serial invalid lengh.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: serial invalid lengh.', fore='green', style='bright'))
        device = None

    try:
        device = ModbusTcpDevice(None)
        print(color('ERROR: serial is None.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: serial is None.', fore='green', style='bright'))

    try:
        device = ModbusTcpDevice(serial)
        print(color('SUCCESS: creating valid device.', fore='green', style='bright'))
    except:
        print(color('ERROR: creating valid device.', fore='red', style='bright'))

    identifier = device.getidentifier()
    if identifier == serial:
        print(color('SUCCESS: checking identifier.', fore='green', style='bright'))
    else:
        print(color('ERROR: checking identifier.', fore='red', style='bright'))

    try:
        setup = device.getsetup()
        if setup == None:
            raise Exception("Setup is None")
        if setup["device"] == "ModbusTcpDevice":
            print(color('SUCCESS: getting device Setup.', fore='green', style='bright'))
        else:
            raise Exception("Setup not valid")
    except Exception as e:
        print(color('ERROR: getting device Setup. Message: ' + str(e), fore='red', style='bright'))

    try:
        data = device.getdata()
        if data == None:
            raise Exception("Data is None")
        if data["check"] == 9544:
            print(color('SUCCESS: getting device data.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except:
        print(color('ERROR: getting device data.', fore='red', style='bright'))

    try:
        logData = device.getlogdata()
        if(logData == None):
            raise Exception("logData is None")
        print(color('SUCCESS: getting device logData.', fore='green', style='bright'))
    except:
        print(color('ERROR: getting device logData.', fore='red', style='bright'))

    try:
        temp = device.getserial()
        if(temp == serial):
            print(color('SUCCESS: getting device Serial.', fore='green', style='bright'))
        else:
            raise Exception("Serial doesn't match serial used to create.")
    except:
        print(color('ERROR: getting device Serial.', fore='red', style='bright'))

    try:
        device.stop()
        print(color('SUCCESS: stopping device before start.', fore='green', style='bright'))
    except:
        print(color('ERROR: stopping device before start.', fore='red', style='bright'))

    try:
        temp = device.getstate()
        if(temp == False):
            print(color('SUCCESS: getting device state (before start).', fore='green', style='bright'))
        else:
            raise Exception("device state is not stopped.")
    except:
        print(color('ERROR: getting device state (before start).', fore='red', style='bright'))

    try:
        temp = device.getdevicetype()
        if(temp == "ModbusTcpDevice"):
            print(color('SUCCESS: getting device type.', fore='green', style='bright'))
        else:
            raise Exception("device type does not match.")
    except:
        print(color('ERROR: getting device type.', fore='red', style='bright'))
 
    device = ModbusTcpDevice(serial_fake)

    try:
        register = device.readregister(1001)
        print(color('ERROR: reading register without modbus client.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading register without modbus client.', fore='green', style='bright'))

    try:
        registers = device.readregisters(1001,10)
        if registers == None:
            print(color('SUCCESS: reading registers without modbus client.', fore='green', style='bright'))
        else:
            print(color('ERROR: reading registers without modbus client.', fore='red', style='bright'))
    except Exception as e:
        print(color('ERROR: reading registers without modbus client. ' + str(e), fore='red', style='bright'))

    try:
        register = device.__readallregisters__()
        print(color('SUCCESS: reading all register without modbus client.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: reading all register without modbus client. ' + str(e), fore='red', style='bright'))

    try:
        device.writeregister(1000, 55)
        print(color('SUCCESS: writing register without modbus client.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: writing register without modbus client.', fore='red', style='bright'))

    try:
        device.sethost(None)
        print(color('ERROR: setting host to none host.', fore='red', style='bright'))        
    except Exception as e:
        print(color('SUCCESS: setting host to none host.', fore='green', style='bright'))

    try:
        device.sethost(123)
        print(color('ERROR: setting host to int host.', fore='red', style='bright'))        
    except Exception as e:
        print(color('SUCCESS: setting host to int host.', fore='green', style='bright'))

    try:
        device.sethost(correctip)
        print(color('SUCCESS: setting host to vaild host.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: setting host to vaild host.', fore='red', style='bright'))
    
    try:
        device.sethost("myhost")
        print(color('SUCCESS: setting host to unreachable host.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: setting host to unreachable host.', fore='red', style='bright'))
    
    try:
        register = device.readregister(1001)
        print(color('ERROR: reading register from unreachable host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading register from unreachable host.', fore='green', style='bright'))

    try:
        registers = device.readregisters(1001,10)
        print(color('ERROR: reading registers from unreachable host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading registers from unreachable host.', fore='green', style='bright'))

    try:
        register = device.__readallregisters__()
        print(color('ERROR: reading all register from unreachable host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading all register from unreachable host.', fore='green', style='bright'))

    try:
        device.writeregister(1000, 55)
        print(color('ERROR: writing register to unreachable host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: writing register to unreachable host.', fore='green', style='bright'))    

    # setting working host
    device = ModbusTcpDevice(serial)

    try:
        register = device.readregister(1001)
        if register >= 0 and register < 1000:
            print(color('SUCCESS: reading register from valid host.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: reading register from valid host. ' + str(e), fore='red', style='bright'))

    try:
        registers = device.readregisters(1000, 10)
        if registers[1001] >= 0 and registers[1001] < 1000:
            print(color('SUCCESS: reading registers from valid host.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: reading registers from valid host. ' + str(e), fore='red', style='bright'))

    try:
        register = device.__readallregisters__()
        print(color('SUCCESS: reading all register from valid host.', fore='green', style='bright'))        
    except Exception as e:
        print(color('ERROR: reading all register from vaild host. ' + str(e), fore='red', style='bright'))

    try:
        device.writeregister(1040, 55)
        print(color('SUCCESS: writing register to valid host.', fore='green', style='bright')) 
    except Exception as e:
        print(color('ERROR: writing register to valid host.', fore='red', style='bright'))

    try:
        device.writeregister(1001, 55)
        print(color('ERROR: writing not writeable register to valid host.', fore='red', style='bright')) 
    except Exception as e:
        print(color('SUCCESS: writing not writeable register to valid host.', fore='green', style='bright')) 

    try:
        host = device.discoverDevice()
        print(color('SUCCESS: discovering device.', fore='green', style='bright')) 
    except Exception as e:
        host = None
        print(color('ERROR: discovering device.', fore='red', style='bright')) 

    if host == correctip:
        print(color('SUCCESS: device discover. ' + str(host), fore='green', style='bright'))
    else:
        print(color('ERROR: device discover. Device not found. Check if in same subnet! Host:' + str(host), fore='red', style='bright')) 

    if(device.getregistervalue(50) == None ):
        print(color('SUCCESS: reading unknown register.', fore='green', style='bright'))
    else:
        print(color('ERROR: reading unknown register.', fore='red', style='bright'))
    
    register = device.getregistervalue(1001)
    if(register != None and register["value"] > 0 and register["value"] < 500):
        print(color('SUCCESS: reading valid register.', fore='green', style='bright'))
    else:
        print(color('ERROR: reading valid register.', fore='red', style='bright'))

    if(device.getdataset("power") == None ):
        print(color('SUCCESS: getting dataset (power) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (power) before processing registers.', fore='red', style='bright'))

    if(device.getlogvalue("power") == None ):
        print(color('SUCCESS: getting logvalue (power) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) before processing registers.', fore='red', style='bright'))

    if(device.getlogvalue("test") == None ):
        print(color('SUCCESS: getting logvalue (test) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) before processing registers.', fore='red', style='bright'))

    device.__processregisters__()

    value = device.getdataset("power")
    if(value != None and value >= 0 ):
        print(color('SUCCESS: getting dataset (power).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (power).', fore='red', style='bright'))

    if(device.getdataset("abc") == None ):
        print(color('SUCCESS: getting dataset (abc).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (abc).', fore='red', style='bright'))

    if(device.getdataset(None) == None ):
        print(color('SUCCESS: getting dataset (None).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (None).', fore='red', style='bright'))

    if(device.getlogvalue(None) == None ):
        print(color('SUCCESS: getting logvalue (None).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (None).', fore='red', style='bright'))

    if(device.getlogvalue("abc") == None ):
        print(color('SUCCESS: getting logvalue (abc).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (abc).', fore='red', style='bright'))

    value = device.getlogvalue("power")
    if(value != None and value == 0 ):
        print(color('SUCCESS: getting logvalue (power).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power).', fore='red', style='bright'))

    device = ModbusTcpDevice(serial)
    device.sethost("dsfjsdljjl")

    try:
        register = device.readregister(1001)
        print(color('ERROR: reading register from invaild host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading register from invaild host.', fore='green', style='bright'))

    try:
        register = device.readregisters(1001, 4)
        print(color('ERROR: reading register from invaild host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading register from invaild host.', fore='green', style='bright'))

    try:
        device.writeregister(1040, 35)
        print(color('ERROR: reading register from invaild host.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: reading register from invaild host.', fore='green', style='bright'))

    device.__supervise__()

    try:
        register = device.readregister(1001)
        print(color('SUCCESS: reading register from valid host after supervice.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: reading register from valid host after supervice.', fore='red', style='bright'))

    try:
        register = device.readregisters(1001, 4)
        print(color('SUCCESS: reading registers from valid host after supervice.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: reading registers from valid host after supervice.', fore='red', style='bright'))

    try:
        device.writeregister(1040, 35)
        print(color('SUCCESS: writing register to valid host after supervice.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: writing register to valid host after supervice.', fore='red', style='bright'))

    device = ModbusTcpDevice(serial)
    device.setmodbusfrequency(0)
    device.start()
    time.sleep(5)

    if( device.getlogvalue("power") == None ):
        print(color('SUCCESS: getting logvalue (power) after starting device with frequency 0.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) after starting device with frequency 0.', fore='red', style='bright'))
    
    if( device.getlogvalue("test") == None ):
        print(color('SUCCESS: getting logvalue (test) after starting device with frequency = 0.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) after starting device with frequency = 0.', fore='red', style='bright'))

    device.setmodbusfrequency(1)
    time.sleep(3)
    device.stop()

    device = ModbusTcpDevice(serial)
    device.start()
    time.sleep(5)
    device.sethost("invalidhost")
    time.sleep(10)

    try:
        register = device.readregister(1001)
        if register >= 0 and register < 1000:
            print(color('SUCCESS: reading register from valid host after discover.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: reading register from valid host after discover. ' + str(e), fore='red', style='bright'))

    device.stop()
    time.sleep(1)

    #DCS communication tests
    device = ModbusTcpDevice(serial)
    connection = DcsConnection(dcsserial, cryptoKey, server, 50333)
    device.addconnection(connection)
    # device.setLogDataSendInterval(5)
    device.start()
    try:
        while True:
            print(color('[ModbusTcpDevice] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
            device.showinfo()
            device.showcommunicationerrors()
            device.showcommunicationerrorsrate()
            time.sleep(10)
    except KeyboardInterrupt as e:
        print("[DCS-Connection] Stopping Test...")
        device.stop()



#!/usr/bin/python

import logging
from datetime import datetime, timedelta
import time
import socket
import random
import json
import threading
import queue
import string
from xtea import MODE_CBC, new
from colr import color

JOINTIMEOUT = 3
RECEIVEMQSITZE = 10
SERIALLEN = 16
CRYPTOKEYLEN = 32
ACKTIMEOUT = 0.5
BUFFERSIZE = 2048
SOCKETRETRIES = 3
SOCKETWAITTIME = 1
KEEPALIVETIMEOUT = 25
RECONNECTTIMEOUT = 5
TIDBLOCKTIME = 90
MAXIMUMSENDTIME = 1.5
MAXIMUMTRANSACTIONTIME = 3
MAXMESSAGEAGE = 5
RECEIVEMSGQUEUSIZE = 2
LOGDATABLOCKTIME = 120

class DcsConnection:
    __serial__ = None
    __cryptoKey__ = None
    __serverAddress__ = None
    __serverPort__ = None
    __socket__ = None
    __connected__ = False
    __receiveBuffer__ = None
    __receiverThread__ = None
    __workerThread__ = None
    __socketLock__ = None
    __threadLock__ = None
    __messageLock__ = None
    __running__ = False
    __sentRequests__ = None
    __ackReceivedEvent__ = None
    __receivedMessageQueue__ = None
    __lastMessageTime__ = None
    __lastMessageSendTime__ = None
    __keepConnected__ = False
    __device__ = None
    __logData__ = None
    __logDataTimestamp__ = None
    
    def __init__(self, serial, cryptoKey, server, port):
        if serial != None and len(serial) == SERIALLEN:
            self.__serial__ = serial
        else:
            errmsg = "Instance not created. Serial is invalid. Serial=" + str(serial)
            logging.error(errmsg)
            raise ValueError(errmsg)

        if cryptoKey != None and len(cryptoKey) == CRYPTOKEYLEN and type(cryptoKey) == str:
            self.__cryptoKey__ = bytearray.fromhex(cryptoKey)
        else:
            errmsg = "Instance not created. CryptoKey invalid. CryptoKey=" + str(cryptoKey)
            logging.error(errmsg)
            raise ValueError(errmsg)

        if server != None:
            self.__serverAddress__ = server
        else:
            errmsg = "Instance not created. Server is required"
            logging.error(errmsg)
            raise ValueError(errmsg)
        
        if port != None and port > 0 and port < 65636:
            self.__serverPort__ = port
        else:
            errmsg = "Instance not created. Port out of range. Port=" + str(port)
            logging.error(errmsg)
            raise ValueError(errmsg)

        self.__socketLock__ = threading.Lock()
        self.__threadLock__ = threading.Lock()
        self.__messageLock__ = threading.Lock()
        self.__sentRequests__ = dict()
        self.__ackReceivedEvent__ = threading.Event()
        self.__receivedMessageQueue__ = queue.Queue(RECEIVEMQSITZE)

        logging.debug("DcsConnection created. Serial: " + str(self.__serial__))

    def __del__(self):
        self.__running__ = False

    def getServer(self):
        return self.__serverAddress__

    def getsockethash(self):
        return self.__socket__.__hash__()

    def addDevice(self, device):
        if(device != None):
            self.__device__ = device
            return True
        else:
            return False

    def isconnected(self):
        return self.__connected__

    def connect(self):
        logging.info(str(id(self)) + " connecting " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + "...")
        if(self.__connected__ == True):
            return True

        self.__lastMessageTime__ = None
        if(not self.__running__):
            self.__start__()
        else:
            logging.debug(str(id(self)) + " threads already running")
        try:
            result = self.__dcsConnect__()
        except Exception as e:
            logging.warning(str(id(self)) + " Failed to open socket " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + ". Server down or network error?")
            self.__keepConnected__ = True
            return False
        if not result:
            logging.warning(str(id(self)) + " Failed to connect " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + ". Check encryption key!")
        else:
            logging.info(str(id(self)) + " Successfully connected " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + ".")
        self.__keepConnected__ = True
        return result

    def disconnect(self):
        self.__keepConnected__ = False
        self.__connected__ = False
        self.__stop__()
        result = self.__dcsDisconnect__()
        if(result):
            logging.info(str(id(self)) + " successfully disconnected")
        else:
            logging.warning(str(id(self)) + "cannont disconnect")
        return result
        
    def reconnect(self):
        self.__connected__ = False
        self.__keepConnected__ = False
        result = self.__dcsDisconnect__()
        if(result):
            logging.info(str(id(self)) + " successfully disconnected. Ready to reconnect")
        else:
            logging.warning(str(id(self)) + "cannont disconnect")
        return self.connect()

    def __createReceiverThread__(self):
        return threading.Thread(target=self.__receive__, name="DCS-RECEIVER "+str(self.__serial__ )+"@"+str(self.__serverAddress__)+":"+str(self.__serverPort__))

    def __createWorkerThread__(self):
        return threading.Thread(target=self.__worker__, name="DCS-WORKER "+str(self.__serial__ )+"@"+str(self.__serverAddress__)+":"+str(self.__serverPort__))

    def __start__(self):
        logging.debug("Threads starting...")
        self.__running__ = True
        self.__receiverThread__ = self.__createReceiverThread__()
        self.__workerThread__ = self.__createWorkerThread__()
        try:
            self.__receiverThread__.start()
            self.__workerThread__.start()
        except Exception as e:
            logging.warning("Threads starting with Exception " + str(e))
    
    def __stop__(self):
        logging.debug("Threads stopping...")
        self.__connected__ = False
        with self.__threadLock__:
            self.__running__ = False
            if(self.__receiverThread__ != None and self.__receiverThread__.is_alive()):
                self.__receiverThread__.join(JOINTIMEOUT)
            logging.debug("Receiver stopped. ")
            if(self.__workerThread__ != None and self.__workerThread__.is_alive()):
                self.__workerThread__.join(JOINTIMEOUT)
            logging.debug("Worker stopped.")

    ##receiving messages
    def __receive__(self):
        logging.debug(str(id(self)) + " Receiver running...")
        while self.__running__ == True:
            wait = False
            with self.__socketLock__:
                if(self.__socket__ != None and not self.__socket__._closed):
                    try:
                        rec = self.__socket__.recv(BUFFERSIZE)
                        self.__receiveBuffer__ += rec
                    except BlockingIOError as e:
                        if e.errno != 10035 and e.errno != 11:
                            logging.error(str(id(self)) + " BlockingIOError: " + str(e))
                    except Exception as e:
                        logging.error(str(id(self)) + " Unknown error receiving data from socket. " + str(e))
                        wait = True     # set wait but don't wait here because mutex has to be released first

            if(wait == True):
                wait = False
                logging.warning(str(id(self)) + " Waiting for socket to recover")
                time.sleep(SOCKETWAITTIME)

            if self.__receiveBuffer__ != None and len(self.__receiveBuffer__)>=32:			
                rec_serialno = self.__receiveBuffer__[0:SERIALLEN].decode("utf-8")
                rec_tid = int.from_bytes(self.__receiveBuffer__[SERIALLEN:18],"little")
                rec_msgtype = int(self.__receiveBuffer__[18])
                rec_paylen = int.from_bytes(self.__receiveBuffer__[19:21],"little")
                rec_payload = self.__receiveBuffer__[32:(32+rec_paylen)]
                self.__receiveBuffer__ = self.__receiveBuffer__[(32+rec_paylen):]

                if(rec_serialno != self.__serial__ ):
                    logging.warning(str(id(self)) + " Invalid serial in received data: " + str(rec_serialno))
                else:
                    logging.debug(str(id(self)) + " Received TID:" + str(rec_tid) + "; Type:" + str(rec_msgtype) + " Paylen: "+str(rec_paylen))
                    message = {
                        "tid": rec_tid,
                        "type": rec_msgtype,
                        "payload": rec_payload,
                        "receivetime": datetime.now()
                    }
                    try:
                        self.__receivedMessageQueue__.put_nowait(message)
                    except Exception as e:
                        logging.warning(str(id(self)) + " cannot put message in __receivedMessageQueue__: " + str(e))
                    self.__lastMessageTime__ = datetime.now()

                    if self.__receivedMessageQueue__.qsize() >= RECEIVEMSGQUEUSIZE:
                        logging.warning(str(id(self)) + " receivedMessageQueue size exceeded. Size: " + str(self.__receivedMessageQueue__.qsize()))
            else:
                logging.debug(str(id(self)) + " nothing to receive.")
                time.sleep(0.1)
 
    #handling massages
    def __worker__(self):
        logging.debug(str(id(self)) + " Worker running...")
        cleanupconter = 0
        while self.__running__ == True:
            if cleanupconter > 50:
                try:
                    self.__cleanSentrequests__()
                    cleanupconter = 0
                except Exception as e:
                    logging.debug(str(id(self)) + " error cleaning mq: " + str(e))
            cleanupconter += 1

            try:
                message = self.__receivedMessageQueue__.get_nowait()
            except Exception:
                message = None
                time.sleep(0.1)

            if(message):
                logging.debug(str(id(self)) + " Handling Message: " + str(message))
                try:
                    difference = datetime.now() - message["receivetime"]
                    if(difference.total_seconds() < MAXMESSAGEAGE or message["type"] == 9):
                        self.__handlemessage__(message["type"], message["payload"], message["tid"])
                        processingtime = datetime.now() - message["receivetime"]
                        if(processingtime.total_seconds() > MAXMESSAGEAGE):
                            logging.warning(str(id(self)) + " processing time too long: " + str(processingtime.total_seconds()) + ". MsgType: " + str(message["type"]) + ", TID: " + str(message["tid"]))
                    else:
                        logging.warning(str(id(self)) + " message to old. Rejecting message. Age: " + str(difference.total_seconds()) + ". MsgType: " + str(message["type"]) + ", TID: " + str(message["tid"]))
                except Exception as e:
                    logging.error(str(id(self)) + " Error handling message. " + str(e))

    def __handlemessage__(self, msgtype, payload, tid):
        if(msgtype == 9):
            logging.debug(str(id(self)) + " TID: " + str(tid) + "; Received ACK")
            try:
                self.__ackrequest__(tid)    #just notify waiting messages and do nothing more
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error handling ACK")
            return

        if(msgtype == 8):
            logging.debug(str(id(self)) + " TID: " + str(tid) + "; Received Keep Alive.")
            try:
                self.__send__(None, 9, tid, isresponce=True)     #send ACK
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending ACK")
            return

        if(msgtype == 4):
            logging.debug(str(id(self)) + " TID: " + str(tid) + "; Received Curent Data Request.")
            try:
                self.senddata(tid=tid)
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending data." +  str(e))
            return

        if(msgtype == 5):
            logging.info(str(id(self)) + " TID: " + str(tid) + "; Received Setup Change.")
            try:
                self.__changeSetup__(payload)
            except Exception as e:
                logging.error(str(id(self)) + " Error applying setup changes. " + str(e))
            try:
                self.sendsetup(tid=tid)
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending setup response")
            return

        if(msgtype == 6):
            logging.info(str(id(self)) + " TID: " + str(tid) + "; Received Calc Log Data (for MSG-Type 2).")
            if payload != None:
                time = self.__decryptPayload__(payload)
            else:
                time = None
            nack=True
            try:
                self.loadlogdata(time)
                nack=False
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + "; Error getting logData. " +  str(e))
            try:
                self.__send__(None, 9, tid, isresponce=True, nack=nack)     #send ACK/NACK
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending ACK/NACK for Calc Log Data. NACK=" + str(nack))
            if nack == False:
                self.clearlogdata()
            return

        if(msgtype == 7):
            logging.info(str(id(self)) + " TID: " + str(tid) + "; Received Send Log Daten (MSG-Type 2).")
            try:
                self.sendlogdata(tid=tid)
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending logData.")
                try:
                    self.__send__(None, 9, tid, isresponce=True, nack=True)     #send NACK
                except Exception as sendexception:
                    logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending NACK for Send Log Data. " + str(sendexception)) 
            return

        logging.info(str(id(self)) + " TID: " + str(tid) + "; Unknown MSG-Type received: " + str(msgtype) + ".")
    
    def watchdog(self):
        logging.debug(str(id(self)) + " DCS-Watchdog running...")
        with self.__threadLock__:
            if(self.__running__ != True):
                logging.warning(str(id(self)) + "DCS-Watchdog: DCS-Connection ist stopped.")
                return self.__running__

            if(self.__receiverThread__ != None and not self.__receiverThread__.is_alive()):
                logging.warning(str(id(self)) + " Receiver thread death. Restarting...")
                try:
                    self.__receiverThread__ = self.__createReceiverThread__()
                    self.__receiverThread__.start()
                except Exception as e:
                    logging.warning(str(id(self)) + " Error restarting receiver Thread: " + str(e))
            
            if(self.__workerThread__ != None and not self.__workerThread__.is_alive()):
                logging.warning(str(id(self)) + " Worker thread death. Restarting...")
                try:
                    self.__workerThread__.join(0.1)
                    self.__workerThread__ = self.__createWorkerThread__()
                    self.__workerThread__.start()
                except Exception as e:
                    logging.warning(str(id(self)) + " Error restarting worker Thread: " + str(e))
            
        if(self.__lastMessageTime__ != None):
            difference = datetime.now() - self.__lastMessageTime__
            if(difference.total_seconds() > KEEPALIVETIMEOUT):
                logging.error(str(id(self)) + " No message received for more then " + str(KEEPALIVETIMEOUT) + " secunds. Reconnecting...")
                try:
                    self.reconnect()
                except Exception as e:
                    logging.warning(str(id(self)) + "Cannot reconect. " + str(e))

        if(self.__keepConnected__ == True and self.__connected__ != True):
            logging.info(str(id(self)) + " disconnection detected. Reconnecting...")
            try:
                self.reconnect()
            except Exception as e:
                logging.warning(str(id(self)) + "Cannot reconect. " + str(e))
        
        try:
            self.__checkSocket__()
        except Exception as e:
                logging.warning(str(id(self)) + "Cannot check socket. " + str(e))

        logging.debug(str(id(self)) + " MSQ-Size: " + str(self.__receivedMessageQueue__.qsize()))
        return self.__running__

    def __cleanSentrequests__(self):
        with self.__messageLock__:
            cleanList = []
            for request in self.__sentRequests__:
                try:
                    if self.__sentRequests__[request] != None and self.__sentRequests__[request]["timestamp"] < datetime.now() - timedelta(seconds=TIDBLOCKTIME):
                        cleanList.append(request)
                except Exception as e:
                    logging.debug(str(id(self)) + " request " + str(request) + " element does not exist anymore. " + str(e))

            for element in cleanList:
                try:
                    logging.debug(str(id(self)) + " Cleaning up request " + str(element) + ".")
                    del self.__sentRequests__[element]
                except Exception as e:
                    logging.debug(str(id(self)) + " Cannot clean request " + str(element) + ". " + str(e))

    def __dcsConnect__(self):
        if not self.__connected__:
            self.__sentRequests__.clear()
            logging.debug(str(id(self)) + " preparing socket.... ")
            if(self.__checkSocket__()):
                try:
                    self.sendsetup(True)
                except Exception as e:
                    logging.error(str(id(self)) + " connection failed. " + str(e))
                    return False

                self.__connected__ = True
                logging.debug(str(id(self)) + " connected " + str(self.__serial__ ))
                return True
            else:
                logging.error(str(id(self)) + " connection failed. No socket available")
                raise Exception("socket error")

    def __checkSocket__(self):
        with self.__socketLock__:
            result = True
            if self.__socket__ == None or self.__socket__._closed:
                result = self.__openSocket__()
            return result

    def __openSocket__(self):
        socketRetries = 0
        logging.debug(str(id(self)) + " No socket. Opening socket...")
        self.__socket__ = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__socket__.settimeout(RECONNECTTIMEOUT)
        self.__receiveBuffer__ = bytes()
        host = ""
        try:
            host = socket.gethostbyname(self.__serverAddress__)
        except Exception:
            logging.error(str(id(self)) + " Host not found.")
            self.__socket__ = None
            return False

        while(socketRetries <= SOCKETRETRIES):
            try:
                logging.debug(str(id(self)) + " trying to connect socket " + str(host) + ":" + str(self.__serverPort__) + "...")
                self.__socket__.connect((host, self.__serverPort__))
                break
            except Exception:
                logging.warning(str(id(self)) + " Cannot open socket to " + str(host) + ":" + str(self.__serverPort__) + ".")
                socketRetries = socketRetries + 1
                time.sleep(SOCKETWAITTIME)
            if socketRetries > SOCKETRETRIES:
                self.__socket__ = None
                
        if(self.__socket__ != None):
            self.__socket__.setblocking(0)
            return True
       
        logging.info(str(id(self)) + " Opening socket failed. Too many retries: " + str(socketRetries))
        return False

    def __dcsDisconnect__(self):
        try:
            if self.__receivedMessageQueue__ != None:
                self.__receivedMessageQueue__.queue.clear()
        except Exception as e:
            logging.error(str(id(self)) + " cannot clear receivedMessageQueue. " + str(e))
            return False
        if self.__socket__ == None or self.__socket__._closed:
            logging.info(str(id(self)) + " Socket is already closed")
            return True
        try:
            self.__closeSocket__()
            logging.info(str(id(self)) + " Socket closed")
            return True
        except Exception as e:
            logging.error(str(id(self)) + " closing socket failed. " + str(e))
            return False    

    def __closeSocket__(self):
        self.__socketLock__.acquire()
        self.__socket__.close()
        self.__socketLock__.release()

    def __ackrequest__(self, tid):
        with self.__messageLock__:
            try:
                self.__sentRequests__[tid]["state"] = "ACK"
                self.__sentRequests__[tid]["timestamp"] = datetime.now()
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. ACK not set. " + str(e))

            try:
                self.__sentRequests__[tid]["event"].set()       #wakeup request
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. Could not wakeup. " + str(e))

    def __send__(self, payload, msgtype, tid, isresponce=False, nack=False):
        startsendtime = datetime.now()
        if self.__socket__ == None or self.__socket__._closed:
            errmsg = str(id(self)) + "Cannot send. No open Socket."
            logging.error(errmsg)
            raise ConnectionError(errmsg)
        try:
            package = self.__make_package__(self.__serial__ , payload, msgtype, tid, nack)
        except Exception as e:
            return False
        if package != None:
            if self.__registermessage__(tid, msgtype, isresponce):
                with self.__socketLock__:
                    try:
                        self.__socket__.send(package)
                        self.__lastMessageSendTime__ = datetime.now()
                        logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " transmitted.")
                    except Exception as e:
                        logging.error(str(id(self)) + " cannot send. " + str(e))
                        return False

                difference = datetime.now() - startsendtime
                if(difference.total_seconds() > MAXIMUMSENDTIME):
                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " sending time to hight! " + str(difference.total_seconds()))
                
                if(isresponce == False):
                    if(not self.__waitForAck__(tid)): 
                        logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " no ACK received.")
                        return False

                difference = datetime.now() - startsendtime
                if(difference.total_seconds() > MAXIMUMTRANSACTIONTIME):
                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " transaction time to hight! " + str(difference.total_seconds()))

                logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " successfull send. Transaction-time: " + str(difference.total_seconds()))
                return True
            else:
                return False
        else:
            logging.error(str(id(self)) + " no package to send")
            return False

    def __registermessage__(self, tid, msgtype, isresponce=False):
        if msgtype == 9 or isresponce==True:
            return True;
        with self.__messageLock__:
            try:
                self.__sentRequests__[tid]
                if self.__sentRequests__[tid] != None:
                    logging.error("[" + self.__serial__ + "] tid " + str(tid) + " already in use")
                    return False
            except Exception:
                pass

            event = threading.Event()
            self.__sentRequests__[tid] = {
                "timestamp": datetime.now(),
                "event": event,
                "state": "NEW"
            }
            return True

    def __waitForAck__(self, tid):
        with self.__messageLock__:
            try:
                if(self.__sentRequests__[tid]["state"] != "ACK"):   #message has already been acked in the while
                    self.__sentRequests__[tid]["state"] = "SEND"
                    self.__sentRequests__[tid]["timestamp"] = datetime.now()
                    event = self.__sentRequests__[tid]["event"]
                else:
                    event = None
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not registrated. " + str(e))
                return False

        if(event):
            event.wait(ACKTIMEOUT)

        response = False
        with self.__messageLock__:
            try:
                if(self.__sentRequests__[tid]["state"] == "ACK"):
                    response = True
                del self.__sentRequests__[tid]
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not found. " + str(e))
            return response

    def __wakeUpWaitingMessages__(self):
        self.__socketLock__.acquire()
        for item in self.__sentRequests__.values():
            temp = item["event"]
            temp.set()
        self.__socketLock__.release()

    def __make_package__(self, device_serno, payl, msgtype, tid, nack=False):
        if(len(device_serno) != SERIALLEN):
            errorMsg = str(id(self)) + " invalid serial " + str(device_serno)
            logging.warning("TID: " + str(tid) + " MakePackage: " + str(errorMsg))
            raise Exception(errorMsg)

        if(msgtype == None):
            errorMsg = str(id(self)) + " no msgtype set "
            logging.warning("TID: " + str(tid) + " MakePackage: " + str(errorMsg))
            raise Exception(errorMsg)

        if(tid == None):
            errorMsg = str(id(self)) + " invalid TID " + str(tid)
            logging.warning("TID: " + str(tid) + " MakePackage: " + str(errorMsg))
            raise Exception(errorMsg)

        if(payl != None):
            try:
                payload = json.dumps(payl).encode('utf-8')
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + " Invalid Payload: " + str(payl) + ". " + str(e))
                return False
        else:
            payload = bytes()
        
        try:
            rand = self.__get_random_string__(8)
            iv = rand.encode('utf-8')
        except Exception as e:
            logging.error("TID: " + str(tid) + " Error generating IV: " + str(e))
            raise(e)
        xtea = new(self.__cryptoKey__, mode = MODE_CBC, IV = iv)

        missing = 8 - len(payload) % 8
        if missing < 8:
            payload = payload + B'\x00' * missing   #always multiple of 8
        if(len(payload) != 0):
            payload = random.randint(0, 2^32).to_bytes(4, byteorder="little")+random.randint(0, 2^32).to_bytes(4, byteorder="little") + payload      # add 8 bytes for initial vector
       
        payload = xtea.encrypt(payload)
        payloadLengh = len(payload).to_bytes(2, byteorder="little")    #calc payload
        Cloudsocket_tid= tid.to_bytes(2, byteorder="little")
        Cloudsocket_msgtype = bytes([msgtype])
        if nack == True:
            ackn = B'\x00'
        else:
            ackn = B'\x01'
        if self.__device__ != None:
            try:
                healthState = self.__device__.getHealthState()
            except Exception as e:
                logging.error("TID: " + str(tid) + " Cannot get healthstate: " + str(e))
                healthState = 0
        else:
            healthState = 0
        healthStateByte = healthState.to_bytes(1, byteorder="little")
        header = device_serno.encode('utf-8') + Cloudsocket_tid + Cloudsocket_msgtype + payloadLengh + ackn + healthStateByte + B'\x00'*9  #build 32 byte header
        package = header + payload
        return  package

    def __get_random_string__(self, length):
        letters = string.ascii_lowercase
        result_str = ''.join(random.choice(letters) for i in range(length))
        return result_str 

    def __decryptPayload__(self, payload):
        iv=payload[(len(payload)-8):]
        xtea = new(self.__cryptoKey__, mode=MODE_CBC,IV=iv)
        payload_decrypted = xtea.decrypt(payload)[8:(len(payload)-2)]
        payload_string = payload_decrypted.decode()
        return payload_string

    def __getsetupelements__(self, payload_string):
        message_string = payload_string[payload_string.find("?")+1:]
        setup_string = message_string[0:message_string.find("/")]
        logging.debug("["+str(self.__serial__ )+"@"+str(self.__serverAddress__)+":"+str(self.__serverPort__)+"]; Setup changes: " + str(setup_string))
        elements = setup_string.split("&")
        return elements

    def senddata(self, tid=None):
        if(self.__connected__):
            try:
                data = self.__getCurrentData__()
            except Exception as e:
                logging.warning(str(id(self)) + " TID: " + str(tid) + ", cannot get CurrentData: " + str(e))
            if(tid == None):
                isresponce=True
                tid = self.__getTid__()
            else:
                isresponce=True
                if(tid > 65535 or tid < 1):
                    raise Exception("TID out of range (1-65535)")
            logging.info(str(id(self)) + " TID: " + str(tid) + ", sending data: " + str(data))
            try:
                if(not self.__send__(data, 1, tid, isresponce=isresponce)):
                    raise Exception("Cannot send data. TID: " + str(tid) + ". " + str(data))
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + ", Error: " + str(e))
                raise
        else:
            errormsg = str(id(self)) + " Error sendig Data. Not connected"
            logging.error(errormsg)
            raise Exception(errormsg)

    def sendlogdata(self, tid=None):
        logData = self.__logData__
        if(logData == None):
            error = str(id(self)) + " Cannot send logData. Empty Logdata."
            logging.warning(error)
            raise Exception(error)

        if(self.__connected__):
            if(tid == None):
                tid = self.__getTid__()
                isresponce = True
            else:
                isresponce = True
                if(tid > 65535 or tid < 1):
                    raise Exception("TID out of range (1-65535)")
            logging.info(str(id(self)) + " TID: " + str(tid) + ", sending logData: " + str(self.__logData__))
            try:
                if(not self.__send__(logData, 2, tid, isresponce=isresponce)):
                    raise Exception("Error sending logData.")
            except Exception as e:
                logging.error(str(id(self)) + " TID: " + str(tid) + ", Error: " + str(e))
                raise
        else:
            errormsg = "Error sendig LogData. Not connected"
            logging.error(errormsg)
            raise Exception(errormsg)

    def __getTid__(self):
        with self.__messageLock__:
            found = None
            while found == None:
                tid = random.randint(1, 65535)
                try:
                    self.__sentRequests__[tid]
                except:
                    found = tid
                    self.__sentRequests__[tid] = None
        return found

    def sendsetup(self, forced = None, tid=None):
        if(self.__connected__ or forced):
            setup = self.__getsetup__()
            if(tid == None):
                tid = self.__getTid__()
                isresponce = False
            else:
                isresponce = True
                if(tid > 65535 or tid < 1):
                    raise Exception("TID out of range (1-65535)")
            logging.info(str(id(self)) + " TID: " + str(tid) + ", sending setup: " + str(setup))
            try:
                if(not self.__send__(setup, 3, tid, isresponce=isresponce)):
                    raise Exception("Error sending setup.")
            except Exception as e:
                logging.error("TID: " + str(tid) + ", Error: " + str(e)) 
                raise
        else:
            errormsg = "Error sendig Setup. Not connected"
            logging.error(errormsg)
            raise Exception(errormsg)

    def __getsetup__(self):
        if(self.__device__ == None):
            logging.warning(str(id(self)) + " device not found. sending dummy setup")
            return {"device": "ConnectionTest", "fwversion": "1.0.0", "serialno": self.__serial__ }
        else:
            return self.__device__.getsetup()

    def __getCurrentData__(self):
        if(self.__device__ == None):
            logging.warning(str(id(self)) + " device not found. getting dummy currData")
            t = datetime.now()
            return {"power": t.second * 100}
        else:
            try:
                return self.__device__.getdata()
            except Exception as e:
                logging.warning(str(id(self)) + " cannot get data. " + str(e))
            
    
    def loadlogdata(self, time = None):

        if self.__logDataTimestamp__ != None:
            difference = datetime.now() - self.__logDataTimestamp__
            if(difference.total_seconds() < LOGDATABLOCKTIME): 
                logging.warning(str(id(self)) + " cannot get logdata. Minimum waiting time not reached. " + str(difference.total_seconds()))
                raise Exception("Logdata not ready.")

        t = datetime.now()
        if time == None:
            time = t.strftime("%Y%m%d%H%M%S")
        else:
            time = str(time)
            
        if(self.__device__ == None):
            logging.warning(str(id(self)) + " device not found. getting dummy logData")
            self.__logData__ = {"i_power": t.second * 100, "time": time}
        else:
            try:
                self.__logData__ = self.__device__.getlogdata(time)
            except Exception as e:
                logging.warning(str(id(self)) + " cannot get logdata. " + str(e))

        
        self.__logDataTimestamp__ = t

    def clearlogdata(self):
        if(self.__device__ == None):
            logging.debug(str(id(self)) + " device not found. nothing to clean.")
        else:
            try:
                self.__device__.clearlog()
            except Exception as e:
                logging.warning(str(id(self)) + " cannot clean logdata.")

    def __changeSetup__(self, payload):
        try:
            setupstring = self.__decryptPayload__(payload)
            setupChanges = self.__getsetupelements__(setupstring)
            logging.debug(str(id(self)) + " Changing setup: " + str(setupChanges))
        except Exception as e:
            error = str(id(self)) + " Error decoding setup changes: " + str(e)
            logging.error(error)
            setupChanges = None
            raise Exception(error)

        if(setupChanges != None):
            for elements in setupChanges:
                setting = elements.split("=")
                field = setting[0]
                value = int(setting[1])

                if(self.__device__ == None):
                    logging.warning(str(id(self)) + " device not found. Settings not applied.")
                    print("Field:" + str(field) + "; Value=" + str(value))
                else:
                    self.__device__.setsetupvalue(field, value)        					

# Entry Point     
if __name__ == "__main__":

    logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)

    serial = "120100200505tes1"
    serial2 = "120100200505tes2"
    serial3 = "120100200505tes3"
    cryptoKey = "41424142414241424142414241424142"
    cryptoKeyInvalid = "ABABABABABABAB00ABABABABABABAB00"
    server = "my-pv.live"

    # if True:
    #     connection = DcsConnection(serial, cryptoKey, server, 50333)
    #     connection2 = DcsConnection(serial2, cryptoKey, server, 50333)
    #     connection.connect()
    #     connection2.connect()
    #     hash1 = connection.getsockethash()
    #     hash2 = connection2.getsockethash()
    #     try:
    #         while True:
    #             print(color('[DCS-Connection] Communication active. Press CTRL+C to stop', fore='blue', style='bright'))
    #             connection.watchdog()
    #             connection2.watchdog()
    #             if(hash1 != connection.getsockethash()):
    #                 print(color('[DCS-Connection] Socket of serial ' + str(serial) + ' has changed.', fore='red', style='bright'))
    #                 hash1 = connection.getsockethash()
    #             if(hash2 != connection2.getsockethash()):
    #                 print(color('[DCS-Connection] Socket of serial ' + str(serial2) + ' has changed.', fore='red', style='bright'))
    #                 hash2 = connection2.getsockethash()

    #             time.sleep(10)
    #     except KeyboardInterrupt as e:
    #         print("[DCS-Connection] Stopping Test...")
    #         connection.disconnect()
    #         connection2.disconnect()

    #AUTO-Tests
    #Constructor Tests
    try:
        DcsConnection("toShort", cryptoKey, server, 50333)
        print(color('ERROR: serial invalid lengh.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: serial invalid lengh.', fore='green', style='bright'))

    try:
        DcsConnection(serial, cryptoKey, server, None)
        print(color('ERROR: Creation without port.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation without port.', fore='green', style='bright'))

    try:
        DcsConnection(serial, cryptoKey, server, 123456789)
        print(color('ERROR: Creation invalid port.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation invalid port.', fore='green', style='bright'))

    try:
        DcsConnection(serial, cryptoKey, None, 50333)
        print(color('ERROR: Creation without server.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation without server.', fore='green', style='bright'))

    try:
        DcsConnection(serial, None, server, 50333)
        print(color('ERROR: Creation without CryptoKey.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation without CryptoKey.', fore='green', style='bright'))

    try:
        DcsConnection(serial, "tooShortKey", server, 50333)
        print(color('ERROR: Creation with to short CryptoKey.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation with to short CryptoKey.', fore='green', style='bright'))

    try:
        DcsConnection(None, cryptoKey, server, 50333)
        print(color('ERROR: Creation without serial.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: Creation without serial.', fore='green', style='bright'))

# Connection Tests
    #connecting to unknown host
    connection = DcsConnection(serial, cryptoKey, "noserver.my-pv.live", 50333)
    if(connection.connect()):
        print(color('ERROR: Connecting to invalid host.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connecting to invalid host.', fore='green', style='bright'))
    connection.disconnect()
    
    #connecting to invalid port
    connection = DcsConnection(serial, cryptoKey, server, 1234)
    if(connection.connect()):
        print(color('ERROR: Connecting to invalid port.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connecting to invalid port.', fore='green', style='bright'))
    connection.disconnect()

    #regular connection
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    if(not connection.connect()):
        print(color('ERROR: Regular connection.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Regular connection.', fore='green', style='bright'))
    connection.disconnect()

    if(not connection.disconnect()):
        print(color('ERROR: disconnection.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: disconnection.', fore='green', style='bright'))
    connection.disconnect()

    #reconnecting
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection.connect()
    if(not connection.reconnect()):
        print(color('ERROR: reconnection.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: reconnection.', fore='green', style='bright'))
    connection.disconnect()

    #connecting to twice
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection.connect()
    if(not connection.connect()):
        print(color('ERROR: Connecting twice.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connecting twice.', fore='green', style='bright'))
    connection.disconnect()

    #connecting after disconnecting
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection.connect()
    connection.disconnect()
    if(not connection.connect()):
        print(color('ERROR: Connecting again.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connecting again.', fore='green', style='bright'))
    connection.disconnect()

    #connecting with invalid cryptoKey
    connection = DcsConnection(serial, "1"*CRYPTOKEYLEN, server, 50333)
    if(connection.connect()):
        print(color('ERROR: Connection with an invalid crypto key esteblished.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connection with an invalid crypto key failed.', fore='green', style='bright'))
    try:
        connection.senddata()
        print(color('ERROR: sending data with invalid encryption Key.', fore='red', style='bright'))
    except Exception as e:
        print(color('ERROR: sending data with invalid encryption Key.', fore='green', style='bright'))
    connection.disconnect()

    #connecting unknown device
    connection = DcsConnection("0123456789ABCEFG", cryptoKey, server, 50333)
    if(connection.connect()):
        print(color('ERROR: Connection using an unknown serial esteblished.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Connection using an unknown serial failed.', fore='green', style='bright'))
    try:
        connection.senddata()
        print(color('ERROR: sending data with unknown device.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: sending data with unknown device.', fore='green', style='bright'))
    connection.disconnect()

    #Starting/Stopping Thread
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    try:
        connection.__start__()
        time.sleep(2)
        connection.__stop__()
        print(color('SUCCESS: Starting/Stopping Thread.', fore='green', style='bright'))
    except Exception as e:
        print(e)
        print(color('ERROR: Starting/Stopping Thread.', fore='red', style='bright'))
    connection.disconnect()

    #sending using valid connection after trying to connect with an invalid connection
    #valid connection
    connection = DcsConnection(serial, cryptoKey, server, 50333) 
    state = connection.connect()
    #connection with invalid key but same serial
    connection2 = DcsConnection(serial, cryptoKeyInvalid, server, 50333) 
    state2 = connection2.connect()
    #send data using first valid connection
    try:
        connection.senddata()
        print(color('SUCCESS: sending using valid connection after trying to connect with an invalid connection.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: sending using valid connection after trying to connect with an invalid connection.', fore='red', style='bright'))
    connection.disconnect()
    connection2.disconnect()

#Communication Tests
    #send without beeing connected
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    try:
        connection.sendsetup()
        print(color('ERROR: sending setup without beeing connected.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: sending setup without beeing connected.', fore='green', style='bright'))

    try:
        connection.senddata()
        print(color('ERROR: sending data without beeing connected.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: sending data without beeing connected.', fore='green', style='bright'))

    try:
        connection.loadlogdata()
        connection.clearlogdata()
        connection.sendlogdata()
        print(color('ERROR: sending logData without beeing connected.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: sending logData without beeing connected.', fore='green', style='bright'))

    try:
        connection.connect()
    except Exception as e:
        print(color('ERROR: Cannot connect for connection tests.', fore='red', style='bright'))

    try:
        connection.senddata()
        print(color('SUCCESS: sending data.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: sending data.', fore='red', style='bright'))

    try:
        print("waiting...")
        time.sleep(LOGDATABLOCKTIME)
        connection.loadlogdata()
        connection.clearlogdata()
        connection.sendlogdata()
        print(color('SUCCESS: sending logData.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: sending logData.', fore='red', style='bright'))

    try:
        connection.loadlogdata()
        print(color('ERROR: getting logData to without waiting.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: getting logData to without waiting.', fore='green', style='bright'))

    try:
        connection.sendsetup()
        print(color('SUCCESS: sending setup.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: sending setup.', fore='red', style='bright'))

    connection.disconnect()

    #sending empty logData
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection.connect()
    try:
        connection.sendlogdata()
        print(color('ERROR: sending empty logData.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: sending empty logData.', fore='green', style='bright'))
    connection.disconnect()

    ##active/passive clients tests
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection.connect()
    socketHash = connection.getsockethash()
    time.sleep(15 + KEEPALIVETIMEOUT)
    if(connection.isconnected() and socketHash == connection.getsockethash()):
        print(color('SUCCESS: Check isconnected.', fore='green', style='bright'))
    else:
        print(color('ERROR: Check isconnected.', fore='red', style='bright'))

    #switch to active client by sending data
    try:
        connection.loadlogdata()
        connection.clearlogdata()
        connection.sendlogdata()
        print(color('SUCCESS: sending unexpected logData after waiting.', fore='green', style='bright'))
    except:
        print(color('ERROR: sending unexpected logData after waiting.', fore='red', style='bright'))
    #switch to active client by sending data   
    try:
        connection.senddata()
        print(color('SUCCESS: sending unexpected currData after waiting.', fore='green', style='bright'))
    except:
        print(color('ERROR: sending unexpected currData after waiting.', fore='red', style='bright'))
    
    time.sleep(10 + KEEPALIVETIMEOUT)
    if(connection.isconnected() and socketHash == connection.getsockethash()):
        print(color('SUCCESS: Check if socket has changed.', fore='green', style='bright'))
    else:
        print(color('ERROR: Check if socket has changed. Old socket-Hash: ' + str(socketHash) + ', new socket-Hash: ' + str(connection.getsockethash()), fore='red', style='bright'))
    connection.disconnect()

    ## watchdog tests
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    try:
        if(connection.watchdog() == False):
            print(color('SUCCESS: Watchdog without started connection.', fore='green', style='bright'))
        else:
            raise Exception("connection running.")
    except Exception as e:
        print(color('ERROR: Watchdog without started connection. ' + str(e), fore='red', style='bright'))
    connection.connect()
    try:
        if(connection.watchdog()):
            print(color('SUCCESS: Watchdog with started connection.', fore='green', style='bright'))
        else:
            raise Exception("connection not running.")
    except Exception as e:
        print(color('ERROR: Watchdog with started connection. ' + str(e), fore='red', style='bright'))
    connection.disconnect()

    time.sleep(10)
    try:
        if(connection.watchdog() == False):
            print(color('SUCCESS: Watchdog with after disconnecting.', fore='green', style='bright'))
        else:
            raise Exception("connection not disconnected.")
    except Exception as e:
        print(color('ERROR: Watchdog with after disconnecting. ' + str(e), fore='red', style='bright'))

    input(color('[DCS-Connection] AUTO-Tests finished. Press enter to continue...', fore='blue', style='bright'))

    connection = DcsConnection(serial, cryptoKey, server, 50333)
    connection2 = DcsConnection(serial2, cryptoKey, server, 50333)
    connection3 = DcsConnection(serial3, cryptoKey, server, 50333)
    connection.connect()
    connection2.connect()
    connection3.connect()
    hash1 = connection.getsockethash()
    hash2 = connection2.getsockethash()
    hash3 = connection3.getsockethash()
    try:
        while True:
            print(color('[DCS-Connection] Communication active. Press CTRL+C to stop', fore='blue', style='bright'))
            connection.watchdog()
            connection2.watchdog()
            connection3.watchdog()
            if(hash1 != connection.getsockethash()):
                print(color('[DCS-Connection] Socket of serial ' + str(serial) + ' has changed.', fore='red', style='bright'))
                hash1 = connection.getsockethash()
            if(hash2 != connection2.getsockethash()):
                print(color('[DCS-Connection] Socket of serial ' + str(serial2) + ' has changed.', fore='red', style='bright'))
                hash2 = connection2.getsockethash()
            if(hash3 != connection3.getsockethash()):
                print(color('[DCS-Connection] Socket of serial ' + str(serial3) + ' has changed.', fore='red', style='bright'))
                hash3 = connection3.getsockethash()
            time.sleep(10)
    except KeyboardInterrupt as e:
        print("[DCS-Connection] Stopping Test...")
        connection.disconnect()
        connection2.disconnect()
        connection3.disconnect()

    time.sleep(1)
    # input(color('[DCS-Connection] Press enter to exit...', fore='blue', style='bright'))



#!/usr/bin/python

from colr import color
from datetime import datetime, timedelta
import logging
import time
import json
import sys
import os

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from mypvdevices.ModbusRtuDevice import ModbusRtuDevice
from mypvdevices.ModbusConnection import ModbusConnection

MODBUSWARNLEVEL = 0.5
REGISTERTIMEOUT = 30
BUILDNR = "4321"

class DeviceDcElwa(ModbusRtuDevice):
    __devicetype__ = "ELWA"
    __irerror__ = False
    __errorcode__ = 0

    def __init__(self, serial, nodeid):
        ModbusRtuDevice.__init__(self, serial, nodeid)
        self.__startRegister__ = 1000
        self.__registersToRead__ = 24

    def __createsetup__(self):
        return {
            "device": self.__devicetype__, \
            "fwversion": self.__firmwareversion__, \
            "serialno": self.__serial__, \
            "elno": self.__nodeId__, \
            "ww2target": None, \
            "ww2offset": 1514
            }

    def __getsettingsmap__(self):
        settings = {
            "ww2target": {
                "register": 1022,   #ELWA Modbus Interface boost temperature
                "forced": False
            },
            "ww2offset": {
                "register": 1023,   #Temp sensor offset calibration
                "forced": True
            }
        }
        return settings

    def getdata(self):

        acRelayState = self.getdataset("ac_relay_state")
        dcPower = self.getdataset("dc_power")
        operationMode = self.getdataset("operation_mode")

        if operationMode == 3 or operationMode == 5 or operationMode == 6 or operationMode == 7 or operationMode == 8 or operationMode == 9 or operationMode == 10 or operationMode == 11 or operationMode == 12 or operationMode == 13 or operationMode == 14 or operationMode == 15 or operationMode == 16 or operationMode == 20 or operationMode == 21 or operationMode == 135:
            acHeating = True
        else:
            acHeating = False

        if acRelayState != None:
            if acHeating == True and acRelayState == 1:
                boostpower = 750
            else:
                boostpower = 0
        else:
            boostpower = None

        if(boostpower != None):
            power = (int(dcPower) + int(boostpower))
            meter = -boostpower
        else:
            power = dcPower
            meter = None

        if self.__errorcode__ != 0:
            errorcode = self.__errorcode__
        else:
            errorcode = None

        errorrate = self.getcommunicationerrorsrate()
        modbusmodulestate = self.getmodbusmodulestate()

        data={
            "device": self.__devicetype__,
            "fwversion": self.__firmwareversion__,
            "loctime": time.strftime("%H:%M:%S"),
            "dev_id": self.__nodeId__,
            "day_counter" : self.getdataset("day_counter"),
            "op_mode": operationMode,
            "dc_breaker": self.getdataset("dc_breaker_state"),
            "dc_relay": self.getdataset("dc_relay_state"),
            "ac_relay": acRelayState,
            "temp1": self.getdataset("temp1"),
            "temp_day_min": self.getdataset("temp_day_min"),
            "temp_day_max": self.getdataset("temp_day_max"),
            "ww1target": self.getdataset("dc_temp_target"),
            "ac_temp_target": self.getdataset("ac_temp_target"),
            "tempchip": self.getdataset("tempchip"),
            "iso_voltage": self.getdataset("iso_voltage"),
            "dc_voltage": self.getdataset("dc_voltage"),
            "dc_current": self.getdataset("dc_current"),
            "power_elwa": power,
            "boostpower_elwa": boostpower,
            "dc_day_wh": self.getdataset("dc_day_wh"),
            "dc_total_kwh": self.getdataset("dc_total_kwh"),
            "ac_day_wh": self.getdataset("ac_day_wh"),
            "minutes_from_noon": self.getdataset("minutes_from_noon"),
            "minutes_since_dusk": self.getdataset("minutes_since_dusk"),
            "ac_boost_mode": self.getdataset("ac_boost_mode"),
            "m1sum": dcPower,
            "m0sum": meter,
            "temp2": self.getdataset("temp2"),
            "ww2target": self.getdataset("boost_temp_target"),
            "ww2offset": self.getdataset("ww2offset_calibration"),
            "modbuserrorrate":  errorrate,
            "modbusmodulestate": modbusmodulestate,
            "errorcode_elwa": errorcode
        }       
        return data

    def getdataset(self, datasetname):
        if self.__irerror__ == False or datasetname in ("temp2"):
            return ModbusRtuDevice.getdataset(self, datasetname)
        else:
            return None

    def getlogdata(self, time = None):
        acRelayState = self.getlogvalue("ac_relay_state")
        if acRelayState != None:
            boostpower = int(round(acRelayState*750))
        else:
            boostpower = None
        
        dcPower = self.getintlogvalue("dc_power")
        if dcPower != None:
            if(boostpower != None):
                power = int(dcPower + boostpower)
            else:
                power = dcPower
        else:
            power = None
        if(boostpower != None):
            metercons = -boostpower
        else:
            metercons = None

        pvprod = dcPower

        sLog={
            "logdataage": self.getlogdataage(),
            "healthstate": self.getHealthState(),
            "errorcode": self.__errorcode__,
            "modbus_if_state": self.getmodbusmodulestate(),
			"modbus_error_rate": self.getcommunicationerrorsrate(),
			"day_counter" : self.getintlogvalue("day_counter"),
			"op_mode": self.getintlogvalue("operation_mode"),
			"dc_breaker": self.getintlogvalue("dc_breaker_state"),
			"dc_relay": self.getintlogvalue("dc_relay_state"),
			"ac_relay": self.getintlogvalue("ac_relay_state"),
			"temp": self.getintlogvalue("temp1"),
			"temp_day_min": self.getintlogvalue("temp_day_min"),
			"temp_day_max": self.getintlogvalue("temp_day_max"),
			"dc_temp_target": self.getintlogvalue("dc_temp_target"),
			"ac_temp_target": self.getintlogvalue("ac_temp_target"),
			"temp_internal": self.getintlogvalue("tempchip"),
			"iso_voltage": self.getintlogvalue("iso_voltage"),
			"dc_voltage": self.getintlogvalue("dc_voltage"),
			"dc_current": self.getintlogvalue("dc_current"),
			"dc_power": self.getintlogvalue("dc_power"),
			"dc_day_wh": self.getintlogvalue("dc_day_wh"),
			"dc_total_kwh": self.getintlogvalue("dc_total_kwh"),
			"ac_day_wh": self.getintlogvalue("ac_day_wh"),
			"minutes_from_noon": self.getintlogvalue("minutes_from_noon"),
			"minutes_since_dusk": self.getintlogvalue("minutes_since_dusk"),
			"ac_boost_mode": self.getintlogvalue("ac_boost_mode"),
			"temp2": self.getintlogvalue("temp2"),
			"ww2target": self.getintlogvalue("boost_temp_target"),
			"ww2offset": self.getintlogvalue("ww2offset_calibration"),
        }

        logData = {
            "time": time,
            "i_power": power,
            "i_boostpower": boostpower,
            "i_m1sum": pvprod,
            "i_metercons": metercons,
            "i_temp1": self.getintlogvalue("temp1"),
            "i_temp2": self.getintlogvalue("temp2"),
            "s_json" : json.dumps(sLog)
        }

        ModbusConnection.instance().resetCounters(self.__nodeId__)
        # self.__logdata__.clear()  #todo sicher?
        return logData

    def getintlogvalue(self, datasetname):
        if self.__irerror__ == False or datasetname in ("temp2"):
            return ModbusRtuDevice.getintlogvalue(self, datasetname)
        else:
            return None

    def getlogvalue(self, datasetname):
        if self.__irerror__ == False or datasetname in ("temp2"):
            return ModbusRtuDevice.getlogvalue(self, datasetname)
        else:
            return None

    def __checkRegiterTimeStamp__(self):
        if self.__registerLastSuccessfullReadTimeStamp__ != None and len(self.__registers__) > 0:
            difference = time.time() - self.__registerLastSuccessfullReadTimeStamp__
            if(difference > REGISTERTIMEOUT):
                return False    
        return True
    
    def __supervise__(self):

        ModbusRtuDevice.__supervise__(self)
        
        value = ModbusRtuDevice.getdataset(self,"temp1")
        if value != None and ( value > 1200 or value < 0 ):
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Temp1 sensor error: " + str(value))
            temp1error = True
        else:
            temp1error = False

        value = ModbusRtuDevice.getdataset(self,"temp2")
        if value != None and ( value > 1200 or value < 0 ):
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Temp2 sensor error: " + str(value))
            temp2error = True
        else:
            temp2error = False

        value = ModbusRtuDevice.getdataset(self,"operation_mode")
        if value != None and value > 100 :
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Operation Mode " + str(value) + " detected.")
            opmodeError = True
        else:
            opmodeError = False

        if ModbusConnection.instance().checkmodbusconnection():
            modbusError = False
        else:
            logging.debug("[DeviceDcElwa] " + self.getidentifier() + " - Modbus module not present.")
            ModbusConnection.instance().resetCounters(self.__nodeId__)
            modbusError = True

        modbusErrorRate = self.getcommunicationerrorsrate()
        
        if modbusError == False:
            if modbusErrorRate != None and modbusErrorRate >= 0.70:
                logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Modbus communication to device not working.")
                modbusError = True
            else:                    
                modbusError = False

        registerisvalid = self.__checkRegiterTimeStamp__()
        if registerisvalid != True and modbusError == False:
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Register values too old. Communication errors expected. Registers: " + str(self.__registers__) + " Timestamp: " + str(self.__registerLastSuccessfullReadTimeStamp__) + " Age: " + str(time.time() - self.__registerLastSuccessfullReadTimeStamp__))
            registerError = True
        else:
            registerError = False

        if registerError == False and modbusError == False and modbusErrorRate != None and modbusErrorRate > MODBUSWARNLEVEL and modbusErrorRate < 0.7:
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - Modbus error rate to hight " + str(modbusErrorRate) + ".")
            modbusWarning = True
        else:
            modbusWarning = False

        value = ModbusRtuDevice.getdataset(self, "tempchip")
        if value == 0 and modbusError == False and registerError == False:
            logging.warning("[DeviceDcElwa] " + self.getidentifier() + " - No IR Connection to Device.")
            irError = True
            self.__irerror__ = True
        else:
            irError = False
            self.__irerror__ = False

        healthState = 0

        #healthState warnings
        if temp2error == True or irError == True or opmodeError == True or modbusWarning == True:
            healthState = 1

        #healthState errors
        if temp1error == True or modbusError == True or registerError == True:
            healthState = 2

        self.__seterrorcode__(irerror=irError, modbuserror=modbusError, registererror=registerError, temp1error=temp1error, temp2error=temp2error, opmodeerror=opmodeError, modbuswarning=modbusWarning)

        if modbusErrorRate != None or healthState == 2:
            self.__sethealthstate__(healthState)

    def __seterrorcode__(self, modbuserror=0, irerror=0, registererror=0, temp1error=0, temp2error=0, opmodeerror=0, modbuswarning=0):
        if modbuswarning: 
            self.__setBit__(0)
        else:
            self.__resetBit__(0)

        if modbuserror:
            self.__setBit__(1)
        else:
            self.__resetBit__(1)

        if irerror:
            self.__setBit__(2)
        else:
            self.__resetBit__(2)

        if registererror:
            self.__setBit__(3)
        else:
            self.__resetBit__(3)

        if temp1error:
            self.__setBit__(4)
        else:
            self.__resetBit__(4)

        if temp2error:
            self.__setBit__(5)
        else:
            self.__resetBit__(5)

        if opmodeerror:
            self.__setBit__(6)
        else:
            self.__resetBit__(6)

    def __setBit__(self, offset):
        mask = 1 << offset
        self.__errorcode__ = (self.__errorcode__ | mask)

    def __resetBit__(self, offset):
        mask = ~(1 << offset)
        self.__errorcode__ = (self.__errorcode__ & mask)

    def __getregistermapping__(self):
        datasets = {}
        datasets["day_counter"] = self.__createDataset__(1000, "avg")
        datasets["operation_mode"] = self.__createDataset__(1001, "avg")
        datasets["dc_breaker_state"] = self.__createDataset__(1002, "avg")
        datasets["dc_relay_state"] = self.__createDataset__(1003, "sum")
        datasets["ac_relay_state"] = self.__createDataset__(1004, "sum")
        datasets["temp1"] = self.__createDataset__(1005, "avg")
        datasets["temp_day_min"] = self.__createDataset__(1006, "avg")
        datasets["temp_day_max"] = self.__createDataset__(1007, "avg")
        datasets["dc_temp_target"] = self.__createDataset__(1008, "avg")
        datasets["ac_temp_target"] = self.__createDataset__(1009, "avg")
        datasets["tempchip"] = self.__createDataset__(1010, "avg")
        datasets["iso_voltage"] = self.__createDataset__(1011, "avg")
        datasets["dc_voltage"] = self.__createDataset__(1012, "avg")
        datasets["dc_current"] = self.__createDataset__(1013, "avg")
        datasets["dc_power"] = self.__createDataset__(1014, "sum")
        datasets["dc_day_wh"] = self.__createDataset__(1015, "avg")
        datasets["dc_total_kwh"] = self.__createDataset__(1016, "avg")
        datasets["ac_day_wh"] = self.__createDataset__(1017, "avg")
        datasets["minutes_from_noon"] = self.__createDataset__(1018, "avg")
        datasets["minutes_since_dusk"] = self.__createDataset__(1019, "avg")
        datasets["ac_boost_mode"] = self.__createDataset__(1020, "avg")
        datasets["temp2"] = self.__createDataset__(1021, "avg")
        datasets["boost_temp_target"] = self.__createDataset__(1022, "avg")
        datasets["ww2offset_calibration"] = self.__createDataset__(1023, "avg")
        return datasets

# Entry Point     
if __name__ == "__main__":

    SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
    sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

    from DcsConnection import DcsConnection

    logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)

    serial = "120100200505tes1"
    cryptoKey = "41424142414241424142414241424142"
    serial2 = "120100200505tes2"
    cryptoKey2 = "41424142414241424142414241424142"
    serial3 = "120100200505tes3"
    cryptoKey3 = "41424142414241424142414241424142"
    server = "my-pv.live"

    # try to read from valid device
    device = DeviceDcElwa(serial, 1)
    try:
        device.__readallregisters__()
        if(len(device.__registers__) == 24):
            print(color('SUCCESS: reading registers.', fore='green', style='bright'))
        else:
            raise Exception("invalid length")
    except Exception as e:
        print(color('ERROR: reading registers. ' + str(e), fore='red', style='bright'))

    try:
        if device.__syncsettings__():
            print(color('SUCCESS: syncing settings.', fore='green', style='bright'))
        else:
            raise Exception("settings sync failed")
    except Exception as e:
        print(color('ERROR: syncing settings. ' + str(e), fore='red', style='bright'))

    try:
        device.__processregisters__()
        value = device.getdataset("dc_power")
        if( value != None ):
            print(color('SUCCESS: processing registers.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: processing registers. ' + str(e), fore='red', style='bright'))

    try:
        value = device.getdataset("seppi")
        if( value == None ):
            print(color('SUCCESS: getting value of unkown registers.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: getting value of unkown registers. ' + str(e), fore='red', style='bright'))

    time.sleep(2)
    device.__processregisters__()
    time.sleep(2)

    device.__readallregisters__()
    try:
        device.__processregisters__()
        value = device.getlogvalue("dc_power")
        if( value != None ):
            print(color('SUCCESS: processing registers again.', fore='green', style='bright'))
        else:
            raise Exception("invalid value")
    except Exception as e:
        print(color('ERROR: processing registers again. ' + str(e), fore='red', style='bright'))

    
    data = device.getdata()
    if(data != None and data != {}):
        print(color('SUCCESS: getting getdata.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting getdata.', fore='red', style='bright'))

    logData = device.getlogdata()
    if(logData != None and logData != {}):
        print(color('SUCCESS: getting getLogData.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting getLogData.', fore='red', style='bright'))

    key = "ww2target"
    targetValue = 534
    try:
        device.setsetupvalue(key, targetValue)
        print(color('SUCCESS: sending setup value to device.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: sending setup value to device.', fore='red', style='bright'))

    setup = device.getsetup()
    if(setup[key] == targetValue):
        print(color('SUCCESS: Setting setup value.', fore='green', style='bright'))
    else:
        print(color('ERROR: Setting setup value.', fore='red', style='bright'))

    if(device != None):
        try:
            register = device.readregister(1021)
            if register > 0 and register < 900: 
                print(color('SUCCESS: reading register.', fore='green', style='bright'))
            else:
                print(color('ERROR: reading register. Value missmatch: ' + str(register), fore='red', style='bright'))
        except Exception as e:
            print(color('ERROR: reading register. Error: ' + str(e), fore='red', style='bright'))

    if(device != None):
        try:
            registers = device.readregisters(1000, 10)
            if registers[1000] !=  0 and registers[1001] != 0:
                print(color('SUCCESS: reading registers.', fore='green', style='bright'))
            else:
                print(color('ERROR: reading registers. Value missmatch: ' + str(registers), fore='red', style='bright'))
        except Exception as e:
            print(color('ERROR: reading registers. Error: ' + str(e), fore='red', style='bright'))
    try:
        device.__supervise__()
        print(color('SUCCESS: supervising.', fore='green', style='bright'))
    except Exception as e:
            print(color('ERROR: supervising. Error: ' + str(e), fore='red', style='bright'))

    input("Press ENTER to start running tests")
    logging.getLogger().setLevel(logging.INFO)
    device = DeviceDcElwa(serial, 1)
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    device.addconnection(connection)
    device.start()

    try:
        while True:
            print(color('[DeviceDcElwa] test active. Press CTRL+C to stop', fore='blue', style='bright'))
            # device.showinfo()
            # device.showcommunicationerrors()
            device.showcommunicationerrorsrate()
            time.sleep(10)
    except KeyboardInterrupt as e:
        print("[DeviceDcElwa] Stopping Test...")
        device.stop()

    input("Press ENTER to start communication tests")

    #DCS communication tests
    logging.getLogger().setLevel(logging.INFO)
    device = DeviceDcElwa(serial, 1)
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    device.addconnection(connection)
    # device.setLogDataSendInterval(5)

    device2 = DeviceDcElwa(serial2, 2)
    connection2 = DcsConnection(serial2, cryptoKey2, server, 50333)
    device2.addconnection(connection2)

    device3 = DeviceDcElwa(serial3, 7)
    connection3 = DcsConnection(serial3, cryptoKey3, server, 50333)
    device3.addconnection(connection3)

    device.start()
    device2.start()
    device3.start()
    try:
        while True:
            print(color('[DeviceDcElwa] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
            # device.showInfo()
            device.showcommunicationerrors()
            device.showcommunicationerrorsrate()
            device2.showcommunicationerrors()
            device2.showcommunicationerrorsrate()
            device3.showcommunicationerrors()
            device3.showcommunicationerrorsrate()
            time.sleep(10)
    except KeyboardInterrupt as e:
        print("[DCS-Connection] Stopping Test...")
        device.stop()
        device2.stop()
        device3.stop()
    input("waiting...")



#!/usr/bin/python

import threading
from pymodbus.client.sync import ModbusSerialClient as ModbusClient
import logging
import time
from colr import color

BUSRECOVERYTIME = 0.5

class ModbusConnection:     #Singleton
    __instance__ = None
    __connectionErrorCounter__ = None
    __connectionCounter__ = None
    __modbusConnection__ = None
    __modbusClient__ = None
    __mutex__ = threading.Lock()
    __modbusModuleState = None
    __modbus_stopbits = 1
    __modbusbytesize = 8
    __modbusparity = 'N'
    __modbusbaudrate = 9600
    __modbustimeout = 0.2
    __modbusretryonempty = True
    __modbusretryoninvalid = True

    @staticmethod
    def instance():
        """ Static access method. """
        if ModbusConnection.__instance__ == None:
            with ModbusConnection.__mutex__:
                ModbusConnection()
        return ModbusConnection.__instance__

    def __init__(self):
        """ Virtually private constructor. """
        if ModbusConnection.__instance__ != None:
            raise Exception("[ModbusConnection] This class is a singleton! Instance already created")
        else:
            ModbusConnection.__instance__ = self

        self.__connectionErrorCounter__ = dict()
        self.__connectionCounter__ = dict()
        self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = self.__modbus_stopbits, bytesize = self.__modbusbytesize, parity = self.__modbusparity, baudrate= self.__modbusbaudrate, timeout=self.__modbustimeout, retry_on_empty=self.__modbusretryonempty, retry_on_invalid=self.__modbusretryoninvalid)

    def __increaseModbusErrorCounter__(self, nodeId):
        try:
            self.__connectionErrorCounter__[nodeId] = self.__connectionErrorCounter__[nodeId] + 1
        except:
            self.__connectionErrorCounter__[nodeId] = 1
        
        logging.debug("[ModbusConnection] Node " + str(nodeId) + ": Communication error. Total errors: " + str(self.__connectionErrorCounter__[nodeId]))
        counter = self.__connectionErrorCounter__[nodeId]
        return counter

    def setConfiguration(self, stopbits, bytesize, parity, baudrate, timeout, retry_on_empty, retry_on_invalid):
        self.__modbus_stopbits = stopbits
        self.__modbusbytesize = bytesize
        self.__modbusparity = parity
        self.__modbusbaudrate = baudrate
        self.__modbustimeout = timeout
        self.__modbusretryonempty = retry_on_empty
        self.__modbusretryoninvalid = retry_on_invalid

        with self.__mutex__:
            self.__modbusClient__.close()
            self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = self.__modbus_stopbits, bytesize = self.__modbusbytesize, parity = self.__modbusparity, baudrate= self.__modbusbaudrate, timeout=self.__modbustimeout, retry_on_empty=self.__modbusretryonempty, retry_on_invalid=self.__modbusretryoninvalid)

    def getModbusErrorCounter(self, nodeId):
        try:
            return self.__connectionErrorCounter__[nodeId]
        except:
            self.__connectionErrorCounter__[nodeId] = 0
            return 0

    def getModbusErrorRate(self, node):
        try:
            self.__connectionErrorCounter__[node]
        except:
            return 0

        try:
            return round(self.__connectionErrorCounter__[node] / self.__connectionCounter__[node], 2)
        except:
            logging.warning("[ModbusConnection] Node " + str(node) + ": Cannot calculate error rate.")
            return None

    def getModbusModuleState(self):
        return self.__modbusModuleState

    def resetCounters(self, nodeId):
        with self.__mutex__:
            try:
                self.__connectionErrorCounter__[nodeId] = 0
                self.__connectionCounter__[nodeId] = 0
            except Exception as e:
                logging.error("[ModbusConnection] Node " + str(node) + ": Error resetting counters. " + str(e))

    def __connectSocket__(self):
        if self.__modbusClient__.socket == None:
            logging.debug("[ModbusConnection] Modbus not connected. Connecting...")
            try:
                self.__modbusConnection__ = self.__modbusClient__.connect()
                if(self.__modbusConnection__ != True):
                    logging.error("[ModbusConnection] Cannot connect modbus. Check modbus module.")
                    raise Exception("Module Error")
            except Exception as e:
                logging.error("[ModbusConnection] Unknown modbus connection error: " + str(e))
                raise(e)
        return self.__modbusConnection__

    def checkmodbusconnection(self):
        return self.__modbusConnection__

    def writeregister(self, node, registerId, value):
        result = None
        with self.__mutex__:
            logging.debug("[ModbusConnection] Node " + str(node) + ": writing register " + str(registerId) +" with value "+ str(value))
            try:
                self.__connectionCounter__[node] += 1
            except:
                self.__connectionCounter__[node] = 1
            if self.__modbusClient__ != None:
                try:
                    socket = self.__connectSocket__()
                except Exception as e:
                    logging.warning("[ModbusConnection] Modbus socket error: " + str(e))
                    socket = None
                if(socket):
                    try:
                        result = self.__modbusClient__.write_register(registerId, value, unit = node)
                        if result.isError():
                            raise Exception(str(result))
                    except Exception as e:
                        logging.warning("[ModbusConnection] Modbus Write Error: " + str(e))
                        self.__increaseModbusErrorCounter__(node)
                        result = None
                else:
                    logging.warning("[ModbusConnection] Cannot write to node " + str(node) + ".")
        return result

    def readregisters(self, node, startAddress = 1000, registersToRead = 1):
        return self.__readregisters__(node, startAddress, registersToRead, "holding")

    def __readregisters__(self, node, startAddress, registersToRead, type="holding"):
        with self.__mutex__:
            logging.debug("[ModbusConnection] Node " + str(node) + ": Reading " + str(registersToRead) + " " + str(type) + "-registers starting with register "+ str(startAddress))
            try:
                self.__connectionCounter__[node] += 1
            except:
                self.__connectionCounter__[node] = 1
            
            if self.__modbusClient__ != None:
                try:
                    socket = self.__connectSocket__()
                except Exception as e:
                    logging.warning("[ModbusConnection] Modbus socket error: " + str(e))
                    socket = None
                if(socket):
                    self.__modbusModuleState = "CONNECTED"
                    registers = dict()
                    try:
                        if type == "input":
                            modbus_response = self.__modbusClient__.read_input_registers(startAddress, registersToRead, unit=node)
                        else:
                            modbus_response = self.__modbusClient__.read_holding_registers(startAddress, registersToRead, unit=node)
                        if modbus_response.isError():
                            raise Exception(str(modbus_response))

                        logging.debug("[ModbusConnection] Node " + str(node) + ": Data: " + str(modbus_response.registers))
                        for i in range(len(modbus_response.registers)):
                            registers[startAddress + i] = modbus_response.registers[i]

                        return registers

                    except Exception as e:
                        logging.debug("[ModbusConnection] Modbus read error at node " + str(node) + ": " + str(e))
                        self.__increaseModbusErrorCounter__(node)
                        time.sleep(BUSRECOVERYTIME)
                else:
                    logging.debug("[ModbusConnection] Cannot read from node " + str(node) + ". Modbus-Socket not connected. Module Error?")
                    self.__modbusModuleState = "ERROR"
            return None

    def readinputregisters(self, node, startAddress = 1000, registersToRead = 1):
        return self.__readregisters__(node, startAddress, registersToRead, "input")


# Entry Point     
if __name__ == "__main__":

    logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)

    unitToTest = 1

    # trying to use modbus module
    try:
        socket = ModbusConnection.instance().__connectSocket__()
    except:
        socket = None
    if(socket == True):
        print(color('SUCCESS: Using modbus module.', fore='green', style='bright'))
    else:
        print(color('ERROR: Using modbus module.', fore='red', style='bright'))
        input("PRESS ENTER TO CONTINUE TESTING")

    # testing module state
    internalTempRegisterNumber = 1021
    register = ModbusConnection.instance().readregisters(unitToTest, internalTempRegisterNumber, 1)
    state =  ModbusConnection.instance().getModbusModuleState()
    if(state == "CONNECTED"):
        print(color('SUCCESS: Modbus module state check.', fore='green', style='bright'))
    else:
        print(color('ERROR: Modbus module state check.', fore='red', style='bright'))

    # trying to read temperature from modbus module internal sensor
    internalTempRegisterNumber = 1021
    register = ModbusConnection.instance().readregisters(unitToTest, internalTempRegisterNumber, 1)
    errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
    errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
    if(register != None and register != {} and errors == 0 and errorrate == 0):
        print(color('SUCCESS: Valid register read of interface temperature.', fore='green', style='bright'))
    else:
        print(color('ERROR: Valid register read of interface temperature failed.', fore='red', style='bright'))

    try:
        if register == None:
            raise(Exception("Empty Register."))
        if(register[internalTempRegisterNumber] < 999 and register[internalTempRegisterNumber] > 0):
            print(color('SUCCESS: Read temperature value in valid range. Temperature: ' + str(register[internalTempRegisterNumber]/10), fore='green', style='bright'))
        else:
            print(color('ERROR: Read temperature value in valid range. OUT OF RANGE: ' + str(register[internalTempRegisterNumber]/10), fore='red', style='bright'))
    except Exception as e:
            print(color('ERROR: Read temperature value in valid range. No data found. ' + str(e), fore='red', style='bright'))

    # reading multiple registers
    ModbusConnection.instance().resetCounters(unitToTest)
    register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
    errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
    errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
    if(register != None and register != {} and errors == 0 and errorrate == 0):
        print(color('SUCCESS: Valid multiple register read.', fore='green', style='bright'))
    else:
        print(color('ERROR: Valid multiple register read.', fore='red', style='bright'))

    # reading multiple registers again
    ModbusConnection.instance().resetCounters(unitToTest)
    register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
    errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
    errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
    if(register != None and register != {} and errors == 0 and errorrate == 0):
        print(color('SUCCESS: Valid multiple register read twice.', fore='green', style='bright'))
    else:
        print(color('ERROR: Valid multiple register read twice.', fore='red', style='bright'))

    # reading an invalid node id
    register = ModbusConnection.instance().readregisters(99, 1000, 24)
    errors = ModbusConnection.instance().getModbusErrorCounter(99)
    if(register == None and errors == 1):
        print(color('SUCCESS: Invalid node read.', fore='green', style='bright'))
    else:
        print(color('ERROR: Invalid node read.', fore='red', style='bright'))

    # reading from broadcast address
    register = ModbusConnection.instance().readregisters(0, 1000, 24)
    errors = ModbusConnection.instance().getModbusErrorCounter(0)
    if(register == None and errors == 1):
        print(color('ERROR: Node 0 (Broadcast) read.', fore='red', style='bright'))
    else:
        print(color('SUCCESS: Node 0 (Broadcast) read.', fore='green', style='bright'))

    # reading of multiple invalid registers
    ModbusConnection.instance().resetCounters(unitToTest)
    register = ModbusConnection.instance().readregisters(unitToTest, 4130, 24)
    errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
    errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
    if(register == None and errors == 1 and errorrate == 1):
        print(color('SUCCESS: Multiple invalid register read.', fore='green', style='bright'))
    else:
        print(color('ERROR: Multiple invalid register read.', fore='red', style='bright'))

    # reading multiple registers multiple times
    ModbusConnection.instance().resetCounters(unitToTest)
    temp = 0
    for i in range(0, 50):
        register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
        errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
        errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
        temp = ModbusConnection.instance().__connectionCounter__[unitToTest]
        time.sleep(0.3)
    if(register != None and register != {} and errors == 0 and errorrate == 0 and temp == 50):
        print(color('SUCCESS: Valid multiple register read multiple times.', fore='green', style='bright'))
    else:
        print(color('ERROR: Valid multiple register read multiple times.', fore='red', style='bright'))

    # trying to change configuration
    internalTempRegisterNumber = 1021
    ModbusConnection.instance().setConfiguration(1, 8, 'N', 19200, 0.2, True, True)     #change to invalid baudrate
    registerInvalid = ModbusConnection.instance().readregisters(unitToTest, internalTempRegisterNumber, 1)
    ModbusConnection.instance().setConfiguration(1, 8, 'N', 9600, 0.2, True, True)  #chnage to valid baudrate
    registerValid = ModbusConnection.instance().readregisters(unitToTest, internalTempRegisterNumber, 1)

    if(registerValid != None and registerValid != {} and registerInvalid == None):
        print(color('SUCCESS: Configuration change test.', fore='green', style='bright'))
    else:
        print(color('ERROR: Configuration change test.', fore='red', style='bright'))

    # writing to vaild node
    ModbusConnection.instance().resetCounters(unitToTest)
    register = 1023
    value = 1514
    try:
        result = ModbusConnection.instance().writeregister(unitToTest, register, value)
        errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
        errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
        if(result != None and result.value == value and errors == 0 and errorrate == 0):
            print(color('SUCCESS: write register to vaild node.', fore='green', style='bright'))
        else:
            print(color('ERROR: write register to vaild node. Invalid value', fore='red', style='bright'))
    except Exception as e:
        print(color('ERROR: write register to vaild node. ' + str(e), fore='red', style='bright'))

    # writing to invaild node
    node = 99
    register = 1023
    value = 1514
    ModbusConnection.instance().resetCounters(node)
    try:
        result = ModbusConnection.instance().writeregister(node, register, value)
        errors = ModbusConnection.instance().getModbusErrorCounter(node)
        errorrate =  ModbusConnection.instance().getModbusErrorRate(node)
        if(result == None and errors == 1 and errorrate == 1):
            print(color('SUCCESS: write register to invaild node.', fore='green', style='bright'))
        else:
            print(color('ERROR: write register to invaild node.', fore='red', style='bright'))
    except Exception as e:
        print(color('ERROR: write register to invaild node. ' + str(e), fore='red', style='bright'))


#!/usr/bin/python

import logging
import time
from datetime import datetime
from colr import color
import sys
import os
import json

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from mypvdevices.ModbusConnection import ModbusConnection
from mypvdevices.ModbusDevice import ModbusDevice

class ModbusRtuDevice(ModbusDevice):
    __devicetype__ = "ModbusRtuDevice"
    __nodeId__ = None

    def __init__(self, serial, nodeid):
        if nodeid != None and isinstance(nodeid, int):
            self.__nodeId__ = nodeid
        else:
            errmsg = "Instance not created. nodeid is invalid. nodeid=" + str(nodeid)
            logging.error(errmsg)
            raise TypeError(errmsg)

        ModbusDevice.__init__(self, serial)

    def getidentifier(self):
        return ModbusDevice.getidentifier(self) + ", Node: " + str(self.__nodeId__)
    
    def __readregister__(self, registerId):
        with self.__lock__:
            try:
                registers = ModbusConnection.instance().readregisters(self.__nodeId__, registerId, 1)
                if registers != None:
                    return registers[registerId]
                else:
                    raise Exception("Modbus communication failed.")
            except Exception as e:
                raise Exception("Register " + str(registerId) + " cannot be read. " + str(e))

    def __readregisters__(self, startregisteraddress, registerstoread):
        with self.__lock__:
            if self.__nodeId__ != None:
                return ModbusConnection.instance().readregisters(self.__nodeId__, startregisteraddress, registerstoread)

    def __writeregister__(self, registerId, valueToWrite):
        with self.__lock__:
            logging.debug("[ModbusRtuDevice] ID " + str(self.__serial__) + " writing register " + str(registerId) + " value: " + str(valueToWrite))
            if self.__nodeId__ != None:
                ModbusConnection.instance().writeregister(self.__nodeId__, registerId, valueToWrite)
            else:
                print("Register " + str(registerId) + " changed to " + str(valueToWrite))

    def getcommunicationerrorscounter(self):
        return ModbusConnection.instance().getModbusErrorCounter(self.__nodeId__)

    def getcommunicationerrorsrate(self):
        return ModbusConnection.instance().getModbusErrorRate(self.__nodeId__)

    def getmodbusmodulestate(self):
        return ModbusConnection.instance().getModbusModuleState()

    def getdata(self):
        t = datetime.now()
        data = {
                "fwversion": self.__firmwareversion__,
				"power": t.second * 100,
                "temp1": t.minute,
                "check": 9811,
                "temp2": self.getdataset("temp2"),
				"loctime": time.strftime("%H:%M:%S")
                }            
        return data

    def getlogdata(self, time=None):
        logdata = super().getlogdata(time=time)
        logdata["s_json"] = json.dumps({
            "modbus_if_state": self.getmodbusmodulestate(),
            "modbus_error_rate": self.getcommunicationerrorsrate(),
        })
        return logdata

# Entry Point     
if __name__ == "__main__":

    from DcsConnection import DcsConnection

    logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)

    #device connection tests
    serial = "120100200505tes1"
    cryptoKey = "41424142414241424142414241424142"
    server = "my-pv.live"

    #AUTO-Tests
    #Constructor Tests
    try:
        device = ModbusRtuDevice("123456789")
        print(color('ERROR: serial invalid lengh.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: serial invalid lengh.', fore='green', style='bright'))
        device = None

    try:
        device = ModbusRtuDevice(None)
        print(color('ERROR: serial is None.', fore='red', style='bright'))
    except:
        print(color('SUCCESS: serial is None.', fore='green', style='bright'))

    try:
        device = ModbusRtuDevice(serial, 1)
        print(color('SUCCESS: creating valid device.', fore='green', style='bright'))
    except:
        print(color('ERROR: creating valid device.', fore='red', style='bright'))

    identifier = device.getidentifier()
    if identifier == serial + ", Node: 1":
        print(color('SUCCESS: checking identifier.', fore='green', style='bright'))
    else:
        print(color('ERROR: checking identifier.', fore='red', style='bright'))

    try:
        setup = device.getsetup()
        if(setup == None):
            raise Exception("Setup is None")
        print(color('SUCCESS: getting device Setup.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: getting device Setup. Message: ' + str(e), fore='red', style='bright'))

    try:
        data = device.getdata()
        if(data == None):
            raise Exception("Data is None")
        print(color('SUCCESS: getting device data.', fore='green', style='bright'))
    except:
        print(color('ERROR: getting device data.', fore='red', style='bright'))

    try:
        logData = device.getlogdata()
        if(logData == None):
            raise Exception("logData is None")
        print(color('SUCCESS: getting device logData.', fore='green', style='bright'))
    except:
        print(color('ERROR: getting device logData.', fore='red', style='bright'))

    try:
        temp = device.getserial()
        if(temp == serial):
            print(color('SUCCESS: getting device Serial.', fore='green', style='bright'))
        else:
            raise Exception("Serial doesn't match serial used to create.")
    except:
        print(color('ERROR: getting device Serial.', fore='red', style='bright'))

    try:
        device.stop()
        print(color('SUCCESS: stopping device before start.', fore='green', style='bright'))
    except:
        print(color('ERROR: stopping device before start.', fore='red', style='bright'))

    try:
        temp = device.getstate()
        if(temp == False):
            print(color('SUCCESS: getting device state (before start).', fore='green', style='bright'))
        else:
            raise Exception("device state is not stopped.")
    except:
        print(color('ERROR: getting device state (before start).', fore='red', style='bright'))

    try:
        temp = device.getdevicetype()
        if(temp == "ModbusRtuDevice"):
            print(color('SUCCESS: getting device type.', fore='green', style='bright'))
        else:
            raise Exception("device type does not match.")
    except:
        print(color('ERROR: getting device type.', fore='red', style='bright'))
 
    #Modbus tests
    device = ModbusRtuDevice(serial, 1)
    try:
        device.__readallregisters__()
        print(color('SUCCESS: reading registers.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: reading registers. ' + str(e), fore='red', style='bright'))

    #register change 
    device.__registers__[1022] = 33
    device.__registers__[1023] = 1234
    try:
        device.__syncsettings__()
        if not device.__setup__["ww2target"] == 33:
            raise Exception("ww2target missmatch")
        if not device.__setup__["ww2offset"] == 1514:
            raise Exception("ww2offset missmatch")
        # if not device.__registers__[1023] == 1514:
        #     raise Exception("register 1023 missmatch")
        print(color('SUCCESS: syncing settings.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: syncing settings. ' + str(e), fore='red', style='bright'))
    
    device.__readallregisters__()

    if(device.getregistervalue(50) == None ):
        print(color('SUCCESS: reading unknown register.', fore='green', style='bright'))
    else:
        print(color('ERROR: reading unknown register.', fore='red', style='bright'))
    
    register = device.getregistervalue(1021)
    if(register != None and register["value"] > 0 and register["value"] < 500):
        print(color('SUCCESS: reading valid register.', fore='green', style='bright'))
    else:
        print(color('ERROR: reading valid register.', fore='red', style='bright'))

    if(device.getdataset("power") == None ):
        print(color('SUCCESS: getting dataset (power) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (power) before processing registers.', fore='red', style='bright'))

    if(device.getlogvalue("power") == None ):
        print(color('SUCCESS: getting logvalue (power) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) before processing registers.', fore='red', style='bright'))

    if(device.getlogvalue("test") == None ):
        print(color('SUCCESS: getting logvalue (test) before processing registers.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) before processing registers.', fore='red', style='bright'))

    device.__processregisters__()

    power = device.getdataset("power")
    if(power != None and power >= 0 ):
        print(color('SUCCESS: getting dataset (power).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (power).', fore='red', style='bright'))

    if(device.getdataset("abc") == None ):
        print(color('SUCCESS: getting dataset (abc).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (abc).', fore='red', style='bright'))

    if(device.getdataset(None) == None ):
        print(color('SUCCESS: getting dataset (None).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting dataset (None).', fore='red', style='bright'))

    if(device.getlogvalue(None) == None ):
        print(color('SUCCESS: getting logvalue (None).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (None).', fore='red', style='bright'))

    if(device.getlogvalue("abc") == None ):
        print(color('SUCCESS: getting logvalue (abc).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (abc).', fore='red', style='bright'))

    if( device.getlogvalue("power") == 0 ):
        print(color('SUCCESS: getting logvalue (power).', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power).', fore='red', style='bright'))

    time.sleep(1)
    device.__readallregisters__()
    device.__registers__[1013] = 700
    device.__processregisters__()
    power = device.getlogvalue("power")
    if( power != None and power > 0 ):
        print(color('SUCCESS: getting logvalue (power) after wait.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) after wait.', fore='red', style='bright'))

    if( device.getlogvalue("test") == 0 ):
        print(color('SUCCESS: getting logvalue (test) after wait.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) after wait.', fore='red', style='bright'))

    if( device.getlogvalue("abc") == None ):
        print(color('SUCCESS: getting logvalue (abc) after wait.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (abc) after wait.', fore='red', style='bright'))

    device.clearlog()
    if( device.getlogvalue("abc") == None ):
        print(color('SUCCESS: getting logvalue (abc) after clear.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (abc) after clear.', fore='red', style='bright'))

    if( device.getlogvalue("test") == None ):
        print(color('SUCCESS: getting logvalue (test) after clear.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) after clear.', fore='red', style='bright'))

    if( device.getlogvalue("power") == None ):
        print(color('SUCCESS: getting logvalue (power) after clear.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) after clear.', fore='red', style='bright'))

    device = ModbusRtuDevice(serial, 1)
    device.setmodbusfrequency(0)
    device.start()
    time.sleep(5)

    if( device.getlogvalue("power") == None ):
        print(color('SUCCESS: getting logvalue (power) after starting device with frequency 0.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (power) after starting device with frequency 0.', fore='red', style='bright'))
    
    if( device.getlogvalue("test") == None ):
        print(color('SUCCESS: getting logvalue (test) after starting device with frequency = 0.', fore='green', style='bright'))
    else:
        print(color('ERROR: getting logvalue (test) after starting device with frequency = 0.', fore='red', style='bright'))

    device.setmodbusfrequency(1)
    time.sleep(3)
    device.stop()

    try:
        device.__sethealthstate__(0)
        print(color('SUCCESS: setting healthState to 0.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: setting healthState to 0.', fore='red', style='bright'))

    try:
        device.__sethealthstate__("seppi")
        print(color('ERROR: setting healthState to invalid.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: setting healthState to invalid.', fore='green', style='bright'))

    try:
        device.__sethealthstate__(None)
        print(color('ERROR: setting healthState to None.', fore='red', style='bright'))
    except Exception as e:
        print(color('SUCCESS: setting healthState to None.', fore='green', style='bright'))

    try:
        device.__sethealthstate__(3)
        print(color('SUCCESS: setting healthState to 3.', fore='green', style='bright'))
    except Exception as e:
        print(color('ERROR: setting healthState to 3.', fore='red', style='bright'))

    if(device != None):
        try:
            register = device.readregister(1021)
            if register > 0 and register < 900: 
                print(color('SUCCESS: reading register.', fore='green', style='bright'))
            else:
                print(color('ERROR: reading register. Value missmatch: ' + str(register), fore='red', style='bright'))
        except Exception as e:
            print(color('ERROR: reading register. Error: ' + str(e), fore='red', style='bright'))

    if(device != None):
        try:
            registers = device.readregisters(1000, 10)
            if registers[1000] !=  0 and registers[1001] != 0:
                print(color('SUCCESS: reading registers.', fore='green', style='bright'))
            else:
                print(color('ERROR: reading registers. Value missmatch: ' + str(registers), fore='red', style='bright'))
        except Exception as e:
            print(color('ERROR: reading registers. Error: ' + str(e), fore='red', style='bright'))

    # device = ModbusRtuDevice(serial, 1)
    # result = device.writeregister(123,22)    #Todo

    #DCS communication tests
    device = ModbusRtuDevice(serial, 1)
    connection = DcsConnection(serial, cryptoKey, server, 50333)
    device.addconnection(connection)
    # device.setLogDataSendInterval(5)
    device.start()
    try:
        while True:
            print(color('[ModbusRtuDevice] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
            device.showinfo()
            device.showcommunicationerrors()
            device.showcommunicationerrorsrate()
            time.sleep(10)
    except KeyboardInterrupt as e:
        print("[DCS-Connection] Stopping Test...")
        device.stop()
